	SUBROUTINE ACTPLT
C
C       This subroutine produces contour plots of a FITS file by means of
C  the general pupose contour plotting subroutine CONTOURPLOT.
C  Contour plots can produced by several different methods;  1) line contours
C  of specified intensity levels, 2) symbol contours of specified intensities,
C  3) line contours spaced equidistant in intensity, 4) scanning, 5) fast
C  and simple scanning.  A file of positions can produced with method 
C  1) if desired.
C
C		Version 1, Arnold Rots, 1973
C		Version 2, B. C. C., 6/10/81
C		Added version, SCRATCH_BLOCK, & _SHARE ENTRY
C               VERSION 3, J. HAYWOOD 5/3/82  U.M.
C               ADDED CALLS TO USRINPG
C		Version 4, D. Gudehus 6/26/82 U.M.
C		Changed DFL (LOGICAL) to CONTOUR (CHARACTER),
C		changed defaults,
C              	added call to TEK,
C		and removed reference to map number.
C		Version 5, Donald H. Gudehus, January, 1984
C		Added calls to COLORMODE.
C		Version 6, Donald H. Gudehus, June, 1988
C		Modularized subroutines, creating new general purpose
C		subroutine CONTOURPLOT.FOR, and added dynamic memory.
C		Version 7, Donald H. Gudehus, October, 1994
C		Added support for SAOimage
C		Version 7U, Donald H. Gudehus, Febrruary 1, 2008
C		Converted to Unix and FITS
C
C Labeling is done in time and arc (RA*DEC)
C INPUT		1-CONTOUR,SX,SY,NCONTOURS,IMI,IMA,JMI,JMA,CFCT
C                        FORMAT A,2F7.3,5I5,F5.0
C                   CONTOUR=D DASHED CONTOURS ALTERNATING WITH SOLID
C                        CONTOURS. (FIRST CONTOUR DASHED)
C                        =S SOLID CONTOUS ONLY
C                   NCONTOURS = NUMBER OF CONTOURS REQUIRED
C                   IMI= STARTING GRIDPOINT IN RA
C                   IMA= END GRIDPOINT IN RA
C                   JMI= STARTING GRIDPOINT IN DEC
C                   JMA= END GRIDPOINT IN DEC
C                   CFCT = FACTOR TO SCALE CONTOURS
C                   C(I) = C(I)*CFCT  (I=1,IC)
C                   DEFAULT: CFCT=1.0
C                 2-C(I),I=1,IC  FORMAT(16F5.0)
C                   C(I)= REQUIRED CONTOUR LEVEL (MAX 10;  larger than this
C			produce overflow of record with .AC)
C                 3-IMX,IMY    FORMAT(2I5)
C                   DMX= DISTANCE ON TICK MARKS IN TDME SEC IN RA-
C                        DIRECTION
C                   DMY= DISTANCE ON TICK MARKS IN ARC SEC IN
C                        DEC-DIRECTION
C                 4-A,D        FORMAT(2F10.5)
C                   A = ALPHA OF STAR
C                   D = DELTA OF STAR
C                    AS MANY STARS AS WANTED ARE PLOTTED, CLOSE
C                    THE SET WITH 0,0
C
	INCLUDE 'miipspar.for'
	INCLUDE 'saofacts.for'
	INCLUDE 'fitsfacts.for'
C
	PARAMETER MAXSIZEDAT=2048
C
	CHARACTER*1 COMMAND
	CHARACTER*1 CONTOUR,OFFSETS_SCALE_SOURCE
	CHARACTER*3 DEVICES(10),PRGFMAT
	CHARACTER*9 XBIN,YBIN
	CHARACTER*40 FILEKEYWORD
	CHARACTER*80 FILENAME_IN
	CHARACTER*132 STRING
	COMMON /CHAR_SCRATCH_BLOCK/ COMMAND,CONTOUR,OFFSETS_SCALE_SOURCE,
	1DEVICES,PRGFMAT,XBIN,YBIN,FILEKEYWORD,FILENAME_IN,STRING
C
	INTEGER STATUS,ADDRESS,FRAMECOLOR,PENWIDTH(10),NSYMBOL(10),COLOR(10),
	3ISZ(4),KIND
C
	LOGICAL IOFL,MORE,
	1PLOTDEVQUES,SUPERPOSE
C  The 64-bit version of the Portland Group compiler gives an error if this
C  comon block is not commented out.
C	COMMON /LOGICAL_SCRATCH_BLOCK/ PLOTDEVQUES,SUPERPOSE
C
	REAL*4 C(10),PAR(2),HEIGHT,S_ARRAY(6000),XYOFF(2),OLDXYOFF(2),
	1XYSCALE(2),OLDXYSCALE(2)
C  ACTPLT and ACTPLT1 should have the same REAL4_SCRATCH_BLOCK2.
	COMMON /REAL4_SCRATCH_BLOCK2/ S_ARRAY
C
	CHARACTER*3 DUMDEVICES(10)
	COMMON /PLOT_CHAR_DEVICES/ DUMDEVICES
	CHARACTER*30 DUMDEVICES2
	EQUIVALENCE (DUMDEVICES,DUMDEVICES2)
C
	LOGICAL GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
	COMMON /DISPLAY_AVAILABILITY/ GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
C
	DIMENSION DATL(MAXSIZEDAT),DATU(MAXSIZEDAT)
	COMMON /CONTOUR_PLOT/ DATL,DATU,DEGPPIXX,DEGPPIXY,RAC,DECC,SCD0,CD0S
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CALL MESSAGE ('ACTPLT, Version 7')
	LU_IN=11
	FILEKEYWORD='FILE='
	FILENAME_IN=' '
	IOFL=.TRUE.
	MAPKEY=0
	PLANEKEY=0
	NMAPS=0
	NPLNS=0
	NROWS=0
	NCOLS=0
	PRGFMAT=' '
	CALL OPENFITS(LU_IN,IOFL,FILEKEYWORD,FILENAME_IN,MAPKEY,PLANEKEY,NMAPS,
	1NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
C  The following are from the VMS version and are not currently used
C	XBIN=CHMHV(12,1,'CTYPE1','RA')
C	YBIN=CHMHV(12,1,'CTYPE2','DEC')
C	RAC=R4MHV(12,1,'CRVAL1',' ')
C	DECC=R4MHV(12,1,'CRVAL2',' ')
C	DEGPPIXX=HED(14) !  Degrees/grid unit in X direction
C	DEGPPIXY=HED(15) !  Degrees/grid unit in Y direction
   1	ISZ(1)=1
	ISZ(2)=NCOLS
	ISZ(3)=1
	ISZ(4)=NROWS
	IZY=0
	MORE=.FALSE.
C	SCP=0.3937
	CALL USRINP(ISZ,4,'I',4,1,'BOUNDARIES=',
	1'Enter start-column end-column start-row end-row:  (default ='//
	1' entire map)')
	IMI=MAX(ISZ(1),1)
	IMA=MIN(ISZ(2),NCOLS)
	JMI=MAX(ISZ(3),1)
	JMA=MIN(ISZ(4),NROWS)
C  Correct for proper reading from the map
	MAPSCOL=IMI
	MAPFCOL=IMA
	MAPSROW=JMI
	MAPFROW=JMA
	NCOL_RD=IMA-IMI+1
	NROW_RD=JMA-JMI+1
C
	KIND=0
	DO WHILE (KIND.NE.1.AND.KIND.NE.2.AND.KIND.NE.3.AND.KIND.NE.4
	1.AND.KIND.NE.5.AND.KIND.NE.6)
	  KIND=1
	  CALL MESSAGE('Enter the kind of plot desired:')
	  CALL MESSAGE('     1 = Contour follower (default)')
	  CALL MESSAGE('     2 = Symbol plotter')
	  CALL MESSAGE('     3 = Equidistant contour plotter')
	  CALL MESSAGE('     4 = Line scanner')
	  CALL MESSAGE('     5 = Fast and simple line scanner')
	  CALL USRINP(KIND,1,'I',4,1,'KIND=',
	1'     6 = "1" + creates a file of positions')
	END DO
C  Get the contour intensity levels
	IF (KIND.EQ.1.OR.KIND.EQ.2.OR.KIND.EQ.4.OR.KIND.EQ.6) THEN
	  CALL USRINP(C,10,'F',4,0,'CONTOURS=',
	1 'Enter the intensity levels for the contours:  (up to 10 allowed)')
	  NCONTOURS=NEL(C,10)
	ELSE IF (KIND.EQ.3) THEN
	  CALL MESSAGE('Contours will be equispaced in intensity;  large'//
	1 ' intensity gradients')
	  CALL MESSAGE('may delete a contour')
	  CALL USRINP(C,10,'F',4,0,'COUNTOURS=',
	1 'Enter the minimun, maximum, and number of levels for the contours:')
	  NCONTOURS=C(3)
	ELSE IF (KIND.EQ.5) THEN
	  CALL USRINP(C,10,'F',4,0,'CONTOURS=',
	1 'Enter a single intensity level for the contour:')
	  NCONTOURS=1
	END IF
	IF (KIND.EQ.2) THEN
	  CALL USRINP(PAR,2,'F',4,0,'RANGES=',
	1 'Enter the intensity range of one symbol and its height in inches:')
	  DELV=PAR(1)
	  HEIGHT=PAR(2)
	  DO I=1,10
	    NSYMBOL(I)=MOD(I,14)-1
	  END DO
	  CALL USRINP(NSYMBOL,10,'I',4,3,'SYMBOL=',
	1 'Enter the symbol types:  (default = 0 1 2, etc.)')
	END IF
C
	COMMAND=' '
	DO WHILE(COMMAND.NE.'A'.AND.COMMAND.NE.'C'.AND.COMMAND.NE.'G'.AND.
	1COMMAND.NE.'S'.AND.COMMAND.NE.'M') 
	  COMMAND='A'
	  CALL MESSAGE('Enter a command for the calculation of offsets'//
	1 ' and scales:')
	  CALL MESSAGE('     A = Automatic selection while preserving'//
	1 ' aspect ratio (default)')
	  CALL MESSAGE('     C = Previously established ACTPLT values')
C	  IF (GRINNELL_270)
C	1 CALL MESSAGE('     G = Previously established Grinnell values')
	  IF (SAOIMAGE)
	1 CALL MESSAGE('     S = Previously established SAOimage values')
	  CALL USRINP(%REF(COMMAND),1,'A',1,1,'COMMAND=','     M ='//
	1 ' Manual selection')
	END DO
	XYOFF(1)=1.4  ! Units of inches
	XYOFF(2)=0.3
	XYSCALE(1)=6.1/FLOAT(NCOL_RD-1)
	XYSCALE(2)=5.3/FLOAT(NROW_RD-1)
	XYSCALE(1)=MIN(XYSCALE(1),XYSCALE(2))
	XYSCALE(2)=XYSCALE(1)
	IF (COMMAND.EQ.'C') THEN
	  XYOFF(1)=OLDXYOFF(1)
	  XYOFF(2)=OLDXYOFF(2)
	  XYSCALE(1)=OLDXYSCALE(1)
	  XYSCALE(2)=OLDXYSCALE(2)
C	ELSE IF (COMMAND.EQ.'G') THEN
C 47.9... = number of pixels/size in inches
C 1.45 = FACTOR from PLTDEV
C	  XYOFF(1)=XLOC(NPIC)/47.90625/1.45
C	  XYOFF(2)=YLOC(NPIC)/47.90625/1.45
C	  XYSCALE(1)=DX(NFRAME,NPIC)/47.90625/1.45
C	  XYSCALE(2)=DY(NFRAME,NPIC)/47.90625/1.45
	ELSE IF (COMMAND.EQ.'S') THEN
C  The offset in SAOimage is controlled in two additive ways.  The first
C  way is by setting SAO_XLOC and SAO_YLOC and sending the information to 
C  SAOimage with SAODELS or SAO_CAPTION.  The second way is by a call to
C  PLOT(X,Y,-3).  Since SAO_XLOC and SAO_YLOC are already set, we only
C  need to make a slight correction with a call to PLOT, via setting
C  XYOFF.
	  XYOFF(1)=2.0/95.0 !  Units of inches;  a 2 pixel adjustment
	  XYOFF(2)=2.0/95.0
	  XYSCALE(1)=1.0/95.0
	  XYSCALE(2)=1.0/95.0
	ELSE IF (COMMAND.EQ.'M') THEN
	  CALL USRINP(XYOFF,2,'F',4,1,'OFFSETS=',
	1 'Enter the X and Y offsets in inches:  (defaults = 1.4 0.3)')  
	  XYSCALE(1)=(6.1-XYOFF(1))/FLOAT(NCOL_RD-1)
	  XYSCALE(2)=(5.3-XYOFF(2))/FLOAT(NROW_RD-1)
	  XYSCALE(1)=MIN(XYSCALE(1),XYSCALE(2))
	  XYSCALE(2)=XYSCALE(1)
	  CALL MESSAGE('Enter the X and Y scales in inches/pixel:')
	  CALL USRINP(XYSCALE,2,'F',4,1,'SCALES=',
	1 '(defaults = maximum while preserving aspect ratio)')
	END IF
	OLDXYOFF(1)=XYOFF(1)
	OLDXYOFF(2)=XYOFF(2)
	OLDXYSCALE(1)=XYSCALE(1)
	OLDXYSCALE(2)=XYSCALE(2)
	DEVICES(1)=' '
	CALL PLOTDEVICES(DEVICES)
	DO I=1,10
	  PENWIDTH(I)=23
	END DO
	IF (.NOT.PLOTDEVQUES('-S',1)) THEN
	  DO I=1,10
	    IF (MOD(I-1,7)+1.EQ.1) COLOR(I)=16776960
	    IF (MOD(I-1,7)+1.EQ.2) COLOR(I)=16711935
	    IF (MOD(I-1,7)+1.EQ.3) COLOR(I)=65280
	    IF (MOD(I-1,7)+1.EQ.4) COLOR(I)=255
	    IF (MOD(I-1,7)+1.EQ.5) COLOR(I)=65535
	    IF (MOD(I-1,7)+1.EQ.6) COLOR(I)=16770229
	    IF (MOD(I-1,7)+1.EQ.7) COLOR(I)=16711680
	  END DO
	ELSE
	  DO I=1,10
	    IF (MOD(I-1,5)+1.EQ.1) COLOR(I)=16777422 !  Yellow, 16777215+207
	    IF (MOD(I-1,5)+1.EQ.2) COLOR(I)=16777419 !  Red
	    IF (MOD(I-1,5)+1.EQ.3) COLOR(I)=16777420 !  Green
	    IF (MOD(I-1,5)+1.EQ.4) COLOR(I)=16777421 !  Blue
	    IF (MOD(I-1,5)+1.EQ.5) COLOR(I)=16777418 !  White
	  END DO
	END IF
	CONTOUR='N'
	IF ((KIND.EQ.1.OR.KIND.EQ.3.OR.KIND.EQ.6).AND.NCONTOURS.GT.1) THEN
	  CALL MESSAGE('Enter the desired contour:')
	  CALL MESSAGE('     N = No dashing (default)')
	  CALL USRINP(%REF(CONTOUR),1,'A',1,1,'CONTOUR=',
	1 '     D = Dashing alternating with solid (first is dashed)')
	END IF
C  Ask about penwidths
	IF (NCONTOURS.GT.1) THEN
	  CALL MESSAGE('Enter the penwidths in thousandths for the '//
	1 'contour(s):')
	  CALL USRINP(PENWIDTH,10,'I',4,3,'PENWIDTHS=',
	1 '(defaults = 23, 23, etc.)')
	  IF (.NOT.PLOTDEVQUES('-S',1)) THEN
	    CALL MESSAGE('Enter the colors for the contour(s):  (defaults =')
	    CALL MESSAGE('yellow, magenta, green, blue, cyan, moccasin, red')
	    CALL MESSAGE('with rollover')
	  ELSE  ! For making overlays with SAOimage, it is better to use
	        ! the overlay colors
	    CALL MESSAGE('When using SAOimage in overlay mode, a restricted'//
	1   'set of colors is available')
	    CALL MESSAGE('Enter the colors for the contour(s):  (defaults =')
	    CALL MESSAGE('yellow, red, green, blue, white with rollover')
	  END IF	  
	  CALL USRINP(COLOR,10,'I',4,3,'COLORS=',' ')
	END IF
	IF (PLOTDEVQUES('G*',1)) THEN
	  CALL MESSAGE('The following commands are available following'//
	1 ' completion of the plot')
	  CALL MESSAGE('a = Allows the user to enter annotation at the'//
	1 ' location of the crosshairs')
	  CALL MESSAGE('    with the color most recently selected')
	  CALL MESSAGE('    ^I (^L on Mac OS) toggles to the Greek'//
	1 ' alphabet and special symbols')
	  CALL MESSAGE('    and back to English')
	  CALL MESSAGE('    ^H (^B on Mac OS) gives a backspace')
	  CALL MESSAGE('    ^K toggles to subscript, superscript, and'//
	1 ' back to normal')
	  CALL MESSAGE('^A = Prompts user for the height of the '//
	1 'annotation')
	  CALL MESSAGE('b = Allows the user to be prompted for the '//
	1 'background color')
	  CALL MESSAGE('C = Turns on the crosshairs and by pressing the'//
	1 ' <CR>, prints the values at')
	  CALL MESSAGE('    that location and puts the values in a'//
	1 ' common block;  typing - deletes the')
	  CALL MESSAGE('    last pair of values from the common block;'//
	1 ' and typing Q turns off the')
	  CALL MESSAGE('    crosshairs.')
	  CALL MESSAGE('c = Allows the user to be prompted for the'//
	1 ' color')
	  CALL MESSAGE('      Overlay colors for SAOimage:')
	  CALL MESSAGE('      Yellow = 16777422, Red = 16777419, '//
	1 'Green  = 16777420')
	  CALL MESSAGE('      Blue = 16777421, White  = 16777418')
 	  CALL MESSAGE('F = Draws a frame around the plot')
	  CALL MESSAGE('H = Help')
	  CALL MESSAGE('h = Gives help')
	  CALL MESSAGE('O = Redoes the original plot')
	  CALL MESSAGE('P = Plots a cross at the given pixel coordinates')
	  CALL MESSAGE('Q = Continues')
	  CALL MESSAGE('q = Continues')
	  CALL MESSAGE('V = Draws a solid vector of the color'//
	1 ' most recently selected')
	  CALL MESSAGE('v = Allows the user to be prompted for the'//
	1 ' starting and ending physical')
	  CALL MESSAGE('    values of a vector')
	END IF
C	IF (PLOTDEVQUES('S*',1)) CALL MESSAGE('This may take a few seconds') !  Not on Unix
C
	CALL DYNAMIC(ADDRESS,NCOL_RD*NROW_RD,'  R4',.TRUE.)
	CALL ACTPLT1(LU_IN,%VAL(ADDRESS),NCOL_RD,NROW_RD,KIND,C,NCONTOURS,DELV,
	1MAPSCOL,MAPSROW,CONTOUR,PENWIDTH,NSYMBOL,COLOR,HEIGHT,XYOFF,XYSCALE)
	CALL DYNAMIC(ADDRESS,NCOL_RD*NROW_RD,'  R4',.FALSE.)
C
C  Ask for more to be plotted
C
	CALL RESETTEK
	MORE=.FALSE.
	CALL USRINP(MORE,1,'L',1,1,'MORE=','Do you wish to do more'//
	1' plotting with this map?  (default = N)')
	SUPERPOSE=.FALSE.
	IF (MORE.AND.PLOTDEVQUES('F',0)) 
	1THEN
	  CALL USRINP(SUPERPOSE,1,'L',1,1,'SUPERPOSE=',
	1 'Do you wish to have superposed images in the plot'//
	2 ' file?  (default = N)')
	END IF
	IF (.NOT.SUPERPOSE) CALL PLOT(0.,0.,999)
	CALL PLOT9999QUES
	IF (MORE) GO TO 1
	CALL CLOSEFITS(LU_IN)
	RETURN
	END
