	SUBROUTINE AVERAGE
C
C       This subroutine averages pixel values within columns, producing 
C  an average for every column for a user specified set of rows in a map or 
C  maps.   Either the average values themselves can be saved (useful for 
C  modeling a CCD frame with upturned left side) or they can be subtracted
C  from the original values (useful for subtracting the sky from a spectrum).
C  The output can be directed to either the input rows or all the rows
C  (default), and to either the original file or a new file of the same size.
C  For future development, LINFIT will be called to perform a polynomial fit 
C  to each column.
C
C		Version 1, Albert Bosma
C		VERSION 2, B. C. C., 7/10/81
C		ADDED VERSION, SCRATCH_BLOCK, & _SHARE ENTRY
C		Version 3, S. Simkin, September, 1981
C		Increased the map size to 2048x2048
C		Version 4, D. Gudehus, January, 1983
C		Fixed up the logic, opened unit 12, added default
C		to all maps, added TV input, and added COMMON blocks.
C		Version 5, J. Haywood and Donald H. Gudehus, April, 1983
C		Added option to exclude data from the average
C		based on their values.
C		Version 6, D. Gudehus, December, 1988
C		Added code for Trapix
C		Version 7, D. Gudehus, May, 1989
C		Added code for subtracting from original if desired.
C		This allowed eliminating the file SKYSUB.FOR.
C		Version 8, D. Gudehus, August, 1993
C		Added code for SAOimage
C		Version 9, D. Gudehus, November 7, 2000
C		Added call to SAO_ERASE_CURSOR so that yellow box would
C		consistently appear after the red one was erased
C		Version 10, Donald H. Gudehus, May 4, 2008
C		Modified code for Unix and FITS

C  Need to add interpolation of ouput average for unaveraged input areas or
C  least squares fitting (better)
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
	INCLUDE 'saofacts.for'
C
	CHARACTER*1 WRITECHOICE,OUTPUTCHOICE
	CHARACTER*2 INPUTMETHOD,CHAR
	CHARACTER*3 FILEFMT
	CHARACTER*40 D1NAME,D2NAME
	CHARACTER*50 CHFITSMHV
	CHARACTER*80 FILENAME_IN
C
	INTEGER*2 IX1,IY1,IX2,IY2
C
	INTEGER MAPS_IN(MAPMAX),MAPS_OUT(MAPMAX),ROWS_IN(MAXSIZE),
	1NOTROW(MAXSIZE),NUM(MAXSIZE),ACTION,CURSOR_TYPE,
	2COLOR_TYPE,STATUS
	INTEGER NCOLS_OUT_ARRAY(MAPMAX),NROWS_OUT_ARRAY(MAPMAX)
C
	LOGICAL EXCLUDE,COLUMNS_DIFFER,ROWS_DIFFER
C
	REAL*4 DAT(MAXSIZE),DATAV(MAXSIZE),DATSUB(MAXSIZE),VALUES(2),INCLINA
C
	REAL*8 DATDP(MAXSIZE),DATAVDP(MAXSIZE),DATSUBDP(MAXSIZE)
C
	CHARACTER*80 DNAME
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	LOGICAL GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
	COMMON /DISPLAY_AVAILABILITY/ GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
C
	CALL MESSAGE ('AVERAGE, Version 10')
	LU_IN=11
	FILENAME_IN=' '
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C
C  Load the arrays for the output sizes.  The output sizes are the same as
C  the input sizes.
C
	DO I=1,NMAPS_IN
	  NCOLS_OUT_ARRAY(I)=NAXIS1(I,LU_IN)
	  NROWS_OUT_ARRAY(I)=NAXIS2(I,LU_IN)
	END DO
C
C  Determine the smallest number of rows and columns in the set of maps
C
	CALL GET_FITS_MAP_LIMITS(LU_IN,NCOLS_MIN,NCOLS_MAX,COLUMNS_DIFFER,
	1NROWS_MIN,NROWS_MAX,ROWS_DIFFER)
C
C  Create a temporary file.  This allows for several options when we save our
C  results.
C
	LU_TEMP=12
	CALL OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS_OUT_ARRAY,NROWS_OUT_ARRAY,STATUS)
C
	INPUTMETHOD=' '
	DO WHILE (INPUTMETHOD.NE.'AS'.AND.INPUTMETHOD.NE.'S'
	1.AND.INPUTMETHOD.NE.'M')
	  CALL MESSAGE('Enter an input method:')
	  INPUTMETHOD='M'
	  IF (SAOIMAGE) THEN
	    CALL MESSAGE('     AS = Rows from cursor;  SAOimage '//
	1   'already loaded')
	    CALL MESSAGE('      S = Rows from cursor;  need to '//
	1   'load SAOimage')
	  END IF
	  CALL MESSAGE('      M = Manual input of rows (default)')
	  CALL USRINP(%REF(INPUTMETHOD),1,'A',2,1,'INPUT_METHOD=',' ')
	  CALL ALPHA(%REF(INPUTMETHOD),1,2,NC,NF,NR,2)
	END DO
	IF (INPUTMETHOD.EQ.'S') THEN
	  NPIC_SAO=1
	  IF (IFB_SAO.EQ.0) THEN
	    CALL SAOSTART
	    CALL SAOFIRST
	  ELSE
	    CALL SAOERAS
	  END IF
	  CALL SAODELS(LU_IN,1,0)
	END IF
	IF (INPUTMETHOD.EQ.'AS'.OR.INPUTMETHOD.EQ.'S') THEN
	  CALL MESSAGE('When positioning the SAOimage cursor as instructed '//
	1 'below, note that the')
	  CALL MESSAGE('X locations are ignored.')
	END IF
C
	IF (INPUTMETHOD.NE.'M') THEN
	  DO J=1,NROWS_MIN
	    K=ISETEND(ROWS_IN(J))
	  END DO
	  NROWSLAST=0
  10	  CHAR=' '
	  IF (INPUTMETHOD.EQ.'AS'.OR.INPUTMETHOD.EQ.'S') THEN
	    MODE=4 !  Gives prompt for 0 to continue, - to delete the 
		   !  last set of rows
C  Get the coordinates and draw an erasable rectangle
	    CALL SAO2CUR(MODE,IX1,IY1,IX2,IY2,X1,Y1,X2,Y2,XM1,YM1,XM2,YM2,
	1   NPIC1,ICHAR,X,Y,a,b)
	    IF (ICHAR.EQ.48) GO TO 200 !  0
	    IF (ICHAR.EQ.45) GO TO 10  !  -
	    NROWS=INT(YM2)-INT(YM1)+1.0
	    IF (NROWS.LE.0.) THEN
	      CALL MESSAGE('Cursors reversed in cursor box')
	      GO TO 10
	    END IF
C  Write vectors onto SAOimage (not erasable)
C  Erase the red box so that the yellow one will appear
	    CALL SAO_ERASE_CURSOR(0)
	    ACTION=2 !  Non erasable cursor
	    CURSOR_TYPE=768 !  Rectangle
	    COLOR_TYPE=207 !  Yellow
	    CALL SAO_DRAW(ACTION,CURSOR_TYPE,COLOR_TYPE,X,Y,NPIC1,
	1   a,b,INCLINA,EXTRA,0)
	  END IF
	  DO J=NROWSLAST+1,NROWSLAST+NROWS
	    ROWS_IN(J)=INT(YM1)+J-NROWSLAST-1
	  END DO
	  NROWSLAST=NROWS+NROWSLAST
	  GO TO 10
	ELSE IF (INPUTMETHOD.EQ.'M') THEN
	  CALL USRINP(ROWS_IN,NROWS_MIN,'I',4,0,'ROWS=',
	1 'Enter the rows for which column averages will be produced:')
	END IF
 200	NROWS_IN=INEL(ROWS_IN,NROWS_MIN)
C  Get the rows not specified and stick them in NOTROW
	NNOT=NROWS_MIN
	CALL EXCEPT(ROWS_IN,NOTROW,NROWS_IN,NNOT,.FALSE.)
	IF (NNOT.GT.0) THEN
	  WRITE(6,3000) (NOTROW(I),I=1,NNOT)
	  WRITE(7,3000) (NOTROW(I),I=1,NNOT)
3000	  FORMAT(' Rows not used '/13(1X,10I4/))
	END IF
C
C  Determine if data is to be excluded
	EXCLUDE=.FALSE.
	CALL MESSAGE('Do you want to exclude data from the averages '//
	1'based on the data''s values?')
	CALL USRINP(EXCLUDE,1,'L',4,1,'EXCLUDE=','(default = N)')
	VALUES(1)=0.0
	VALUES(2)=16383.0
	IF (EXCLUDE) THEN
	  CALL MESSAGE('Enter the lower and upper limits at and beyond '//
	1 'which data will be excluded:')
	  CALL USRINP(VALUES,2,'F',4,1,'VALUES=',
	1 '(defaults = 0.0 16383.0)')
	END IF
C  Determine whether to write averages to all the rows or just the input row
C  numbers
	OUTPUTCHOICE='A'
	CALL MESSAGE('Enter a choice for the output value:')
	CALL MESSAGE('     A = Output is average (default)')
	CALL USRINP(%REF(OUTPUTCHOICE),1,'A',1,1,'OUTPUT_CHOICE=',
	1'     B = Output is original - average')
	IF (NNOT.GT.0) THEN
	  WRITECHOICE='B'
	  CALL MESSAGE('Enter a choice for the ouput file:')
	  CALL MESSAGE('     A = Writes output to all rows')
	  CALL USRINP(%REF(WRITECHOICE),1,'A',1,1,'WRITE_CHOICE=',
	1 '     B = Writes output to input rows;  noninput rows remain '//
	2 'the same (default)')
	END IF
C
	DO M=1,NMAPS_IN !  Loop for maps
	  FILEFMT=CHFITSMHV(LU_IN,IM,'BITPIX','R4')
	  DO IP=1,NAXIS3(M,LU_IN) !  Loop for planes
	    IF (FILEFMT.EQ.'R8'.OR.FILEFMT.EQ.'C16') THEN
	      CALL ZERODP(DATAVDP,NCOLS_MAX)
	      CALL IZERO(NUM,NCOLS_MAX)
	      DO J=1,NROWS_IN !  Loop for input rows
	        CALL RDDPFITSROW(LU_IN,MAPS_IN(M),IP,DATDP,
	1       NCOLS_MAX,ROWS_IN(J))
	        IF (EXCLUDE) THEN !  Faster to put IF outside loop
	          DO I=1,NCOLS_OUT_ARRAY(M) !  Loop for columns
	            IF (DATDP(I).GT.VALUES(1).AND.DAT(I).LT.VALUES(2)) THEN
	              DATAVDP(I)=DATAVDP(I)+DATDP(I)
	              NUM(I)=NUM(I)+1
	            END IF
	          END DO
	        ELSE
	          DO I=1,NCOLS_OUT_ARRAY(M)
	            DATAVDP(I)=DATAVDP(I)+DATDP(I)
	            NUM(I)=NUM(I)+1
	          END DO
	        END IF
	      END DO
C  Calculate the averages
              DO I=1,NCOLS_OUT_ARRAY(M)
               IF (NUM(I).NE.0) DATAVDP(I)=DATAVDP(I)/DBLE(NUM(I))
              END DO
C  Write the averages or subtractions, to the input rows (of the output map)
C  first
	      DO J=1,NROWS_IN
	        IF (OUTPUTCHOICE.EQ.'A') THEN !  Output is average
	          CALL WRDPFITSROW(LU_TEMP,MAPS_IN(M),IP,DATAVDP,NCOLS_MAX,
	1         ROWS_IN(J))
	        ELSE IF (OUTPUTCHOICE.EQ.'B') THEN !  Output is subtraction
	          CALL RDDPFITSROW(LU_IN,MAPS_IN(M),IP,DATDP,NCOLS_MAX,
	1         ROWS_IN(J))
	          DO I=1,NCOLS_OUT_ARRAY(M)
	            DATSUBDP(I)=DATDP(I)-DATAVDP(I)
	          END DO
	          CALL WRDPFITSROW(LU_TEMP,MAPS_IN(M),IP,DATSUBDP,NCOLS_MAX,
	1         ROWS_IN(J))
	        END IF
	      END DO
C  Write to the non-input rows as requested
	      DO J=1,NNOT
	        IF (WRITECHOICE.EQ.'A') THEN !  Writes to all rows
	          IF (OUTPUTCHOICE.EQ.'A') THEN !  Writes average value
	            CALL WRDPFITSROW(LU_TEMP,MAPS_IN(M),IP,DATAVDP,NCOLS_MAX,
	1           NOTROW(J))
	          ELSE IF (OUTPUTCHOICE.EQ.'B') THEN !  Subtracts from original
	            CALL RDDPFITSROW(LU_IN,MAPS_IN(M),IP,DATDP,NCOLS_MAX,
	1           NOTROW(J))
	            DO I=1,NCOLS_OUT_ARRAY(M) !  Does the whole row
	              DATSUBDP(I)=DATDP(I)-DATAVDP(I)
	            END DO
	            CALL WRDPFITSROW(LU_TEMP,MAPS_IN(M),IP,DATSUBDP,NCOLS_MAX,
	1           NOTROW(J))
	          END IF
	        ELSE IF (WRITECHOICE.EQ.'B') THEN
C  No need to do anything
c	        ELSE IF (WRITECHOICE.EQ.'B'.AND.
c	1       ((MAPS_IN(M).NE.MAPS_IN(M).OR.D1NAME.NE.D2NAME))) THEN
c	!  Copies over the original rows if the maps or files are
c	!  different
c	          CALL RDDPFITSROW(LU_IN,MAPS_IN(M),IP,DATDP,NCOLS_MAX,
c	1         NOTROW(J))
c	          CALL WRDPFITSROW(LU_TEMP,MAPS_IN(M),IP,DATDP,NCOLS_MAX,
c	1         NOTROW(J))
	        END IF
	      END DO




	    ELSE
	      CALL ZERO(DATAV,NCOLS_MAX)
	      CALL IZERO(NUM,NCOLS_MAX)
	      DO J=1,NROWS_IN !  Loop for input rows
	        CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,DAT,NCOLS_MAX,ROWS_IN(J))
	        IF (EXCLUDE) THEN !  Faster to put IF outside loop
	          DO I=1,NCOLS_OUT_ARRAY(M) !  Loop for columns
	            IF (DAT(I).GT.VALUES(1).AND.DAT(I).LT.VALUES(2)) THEN
	              DATAV(I)=DATAV(I)+DAT(I)
	              NUM(I)=NUM(I)+1
	           END IF
	          END DO
	        ELSE
	          DO I=1,NCOLS_OUT_ARRAY(M)
	            DATAV(I)=DATAV(I)+DAT(I)
	            NUM(I)=NUM(I)+1
	          END DO
	        END IF
	      END DO
C  Calculate the averages
	      DO I=1,NCOLS_OUT_ARRAY(M)
	        IF (NUM(I).NE.0) DATAV(I)=DATAV(I)/FLOAT(NUM(I))
	      END DO
C  Write the averages or subtractions, to the input rows (of the output map) 
C  first
	      DO J=1,NROWS_IN
	        IF (OUTPUTCHOICE.EQ.'A') THEN !  Output is average
	          CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,DATAV,NCOLS_MAX,
	1         ROWS_IN(J))
	        ELSE IF (OUTPUTCHOICE.EQ.'B') THEN !  Output is subtraction
	          CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,DAT,NCOLS_MAX,
	1         ROWS_IN(J))
	          DO I=1,NCOLS_OUT_ARRAY(M)
	            DATSUB(I)=DAT(I)-DATAV(I)
	          END DO
	          CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,DATSUB,NCOLS_MAX,
	1         ROWS_IN(J))
	        END IF
	      END DO
C  Write to the non-input rows as requested
	      DO J=1,NNOT
	        IF (WRITECHOICE.EQ.'A') THEN !  Writes to all rows
	          IF (OUTPUTCHOICE.EQ.'A') THEN !  Writes average value
	            CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,DATAV,NCOLS_MAX,
	1           NOTROW(J))
	          ELSE IF (OUTPUTCHOICE.EQ.'B') THEN !  Subtracts from original
	            CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,DAT,NCOLS_MAX,
	1           NOTROW(J))
	            DO I=1,NCOLS_OUT_ARRAY(M) !  Does the whole row
	              DATSUB(I)=DAT(I)-DATAV(I)
	            END DO
	            CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,DATSUB,NCOLS_MAX,
	1           NOTROW(J))
	          END IF
	        ELSE IF (WRITECHOICE.EQ.'B') THEN
C  No need to do anything
c	        ELSE IF (WRITECHOICE.EQ.'B'.AND.
c	1       ((MAPS_IN(M).NE.MAPS_IN(M).OR.D1NAME.NE.D2NAME))) THEN
c		!  Copies over the original rows if the maps or files are 
c	 	!  different
c	          CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,DAT,NCOLS_MAX,
c	1         NOTROW(J))
c	          CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,DAT,NCOLS_MAX,
c	1         NOTROW(J))
	        END IF
	      END DO
	    END IF
	  END DO !  Ends loop for planes
	END DO !  Ends loop for maps
C
C  If SAOimge was used, erase the cursor
	IF (INPUTMETHOD.EQ.'AS'.OR.INPUTMETHOD.EQ.'S') THEN
	  CALL SAO_ERASE_CURSOR(0)
	END IF
C
	LU_OUT=13
	CALL SAVE_SAVE_AS(LU_IN,LU_TEMP,LU_OUT,MAPS_IN,NMAPS_IN,STATUS)
C  The files are closed by the above call to save_save_as.
 999	CONTINUE
	RETURN
	END
