	SUBROUTINE FIND
C
C       This subroutine can locate objects in a FITS file which has been
C  optionally filtered with options PW, MT, and RT.  The locations of the
C  peaks in the map, the sky values, and the peak values are written to a 
C  character file of type 2 (prepared by option CT or this option).  The 
C  objects' ID are written as numbers.  The object positions can be input
C  with the TV cursor or automatically.  Option CT can then operate on the
C  original map using the starting values from the data file created by
C  this option.  One can also create an artificial file of type 2 containing
C  star positions and other data.
C				Donald H. Gudehus, January, 1987
C
C			Version 2, Donald H. Gudehus, June, 1989
C			Added code for creating artificial star data and
C			to use Trapix.
C			Version 3, Donald H. Gudehus, August, 1993
C			Added code for SAOimage and changed call to
C			TVMAPCIR.
C			Version 3U, Donald H. Gudehus, February 15, 2008
C			Converted to Unix and FITS, and removed code for
C			the Grinnell and Trapix.
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
	INCLUDE 'saofacts.for'
C
	CHARACTER*2 COMMAND
	CHARACTER*3 PRGFMAT
	CHARACTER*40 FILEKEYWORD
	CHARACTER*80 FILENAME
C
	INTEGER*2 IXN,IYN
	INTEGER MAPKEY,PLANEKEY,STATUS
	INTEGER CURSOR_TYPE,COLOR_TYPE,ACTION,NPICN,KEY,ICOUNT,IC,IQ(4),
	1ISIZE(4),ADDRESS,ADDRESS2,OBJDIAM,NII,NJJ,LENGTH,NUMBER_STARS,
	2ISEED
C
	LOGICAL MESS,ERASE
C
	REAL RADIUS,INCLINA,DATA,BOUNDARIES(4),INCLINATION,
	1ECCENTRICITY,FWHMA,FWHMB,FWHM(2),XN,YN,XMN,YMN,SKY,SKY_PARAMETERS,
	2K_SIGMA,XDELTA,YDELTA,ROWARRAY(MAXSIZE),ADU_RANGE(2),ADUDELTA
C
	CHARACTER*6 ID
	COMMON /ID/ ID
C
	CHARACTER*80 DNAME
	LOGICAL EXTENDED
	COMMON /DFILE/ DNAME,EXTENDED
C
	LOGICAL OFIL
	COMMON /OLDFILE/ OFIL
C
	LOGICAL GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
	COMMON /DISPLAY_AVAILABILITY/ GRINNELL_270,TRAPIX,RAMTEK,SAOIMAGE
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	COMMAND=' '
	DO WHILE (COMMAND.NE.'AS'.AND.COMMAND.NE.'S'.AND.
	2COMMAND.NE.'F'.AND.COMMAND.NE.'M') 
	  CALL MESSAGE('Enter a command:')
	  COMMAND='F'
	  IF (SAOIMAGE) THEN
	    CALL MESSAGE('     AS = Position from cursor;  SAOimage '//
	1   'already loaded')
	    CALL MESSAGE('      S = Position from cursor;  need to '//
	1   'load SAOimage')
	  END IF
	  CALL MESSAGE('      F = Automatic location of objects (default)')
	  CALL MESSAGE('      M = Monte Carlo simulation of positions')
	  CALL USRINP(%REF(COMMAND),1,'A',2,1,'COMMAND=',' ')
	  CALL ALPHA(%REF(COMMAND),1,2,NC,NF,NR,2)
	END DO
	IF (COMMAND.NE.'M') THEN
	  CALL MESSAGE('Open the FITS file with objects to be found')
	  LU_IN=11
	  FILEKEYWORD='FILE='
	  FILENAME=' '
	  IOFL=.TRUE.
	  MAPKEY=0
	  PLANEKEY=0
	  NMAPS=0
	  NPLNS=0
	  NROWS=0
	  NCOLS=0
	  PRGFMAT=' '
	  CALL OPENFITS(LU_IN,IOFL,FILEKEYWORD,FILENAME,MAPKEY,PLANEKEY,NMAPS,
	1 NPLNS,NCOLS,NROWS,PRGFMAT,STATUS)
	  IF (NCOLS.GT.MAXSIZE) THEN
	    CALL ERROR(-3,'Row length is too large for present '//
	1   'dimension in FIND')
	    RETURN
	  END IF
	END IF
	CALL MESSAGE('Open the output character file of type 2')
	CALL OPENCF(21,0,'FILE_21=',0) 
	IF (OFIL) THEN
	  REWIND 21
	  READ (21,1200)
1200	  FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',2X,'INCLINATION',
	1 1X,'ECCENTRICITY',1X,'FWHMA',5X,'FWHMB',1X,
	1 'QUADRANT CODE        PEAK             SKY')
	  DO WHILE(.TRUE.)
	    READ(21,1000,END=100) ICOUNT
1000	    FORMAT(' ',I6)
	  END DO
	ELSE
	  ICOUNT=0
	  WRITE(21,1200)
 100	END IF
C  Set up some defaults
	INCLINATION=0.0
	ECCENTRICITY=0.0
	FWHMA=0.0
	FWHMB=0.0
	DO I=1,4
	  IQ(I)=1
	END DO
	MESS=.TRUE.
C Begin the main IF block
	IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
	  IF (COMMAND.EQ.'S') THEN
	    NPIC_SAO=1
	    IF (IFB_SAO.EQ.0) THEN
	      CALL SAOSTART
	      CALL SAOFIRST
	    ELSE
	      CALL SAOERAS
	    END IF
	    CALL SAODELS(LU_IN,1,0)
	  END IF
	  CALL USRINP(SKY,1,'F',4,0,'SKY=','Enter a value for the sky:')
	  ACTION=11 !  For SAOimage
	  DO WHILE(.TRUE.)
	    IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
  55	      CALL MESSAGE('Be sure SAOimage has focus, position the '//
	1     'cursor, and then press <CR>')
	      CALL MESSAGE('(type 0 or Q to quit)')
	      CALL SAO_CURN(NPICN,IXN,IYN,XN,YN,XMN,YMN,KEY,MESS)
	      IF (KEY.EQ.13) THEN
	        IF (NPICN.EQ.0.OR.(NPICN.NE.0.AND.TV_FILENAME(NPICN)
	1       .NE.DNAME)) THEN
	          CALL MESSAGE('Try again')
	          GO TO 55
	        END IF
	        WRITE(6,1100) INT(XMN),INT(YMN)
1100	        FORMAT(' ','Cursor is at ',I6,1X,I6,' in the map')
	        WRITE(7,1100) INT(XMN),INT(YMN)
	        CURSOR_TYPE=1024
	        COLOR_TYPE=207 !  Yellow
	        COLUMN=INT(XMN)
	        ROW=INT(YMN)
	        RADIUS=4.0
	        INCLINA=0.0
	        CALL SAO_DRAW(ACTION,CURSOR_TYPE,COLOR_TYPE,COLUMN+0.5,ROW+0.5,
	1       NPICN,RADIUS,RADIUS,INCLINA,EXTRA,0) 
	      ELSE IF (KEY.EQ.48.OR.KEY.EQ.81) THEN
	        GO TO 900
	      ELSE
	        CALL BELL
	        CALL MESSAGE('You must press the <CR>, 0, or Q!')
	        GO TO 55
	      END IF
	    END IF
	    CALL RDFITSXY(LU_IN,IMNUM,IPNUM,DATA,1,1,INT(XMN),1,INT(YMN),1)
	    ICOUNT=ICOUNT+1
	    IC=INT(LOG10(REAL(ICOUNT)))+1
	    ENCODE(6,2000,ID) ICOUNT
2000	    FORMAT(I<IC>)
	    WRITE(21,1201) ID,XMN,YMN,INCLINATION,ECCENTRICITY,
	1   FWHMA,FWHMB,(IQ(JJ),JJ=1,4),DATA,SKY
1201	    FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,1X,F9.3,1X,
	1   F9.3,3X,4I2,5X,G16.8,1X,G16.8)
	  END DO
	ELSE IF (COMMAND.EQ.'F') THEN
	  ISIZE(1)=1
	  ISIZE(2)=NCOLS
	  ISIZE(3)=1
	  ISIZE(4)=NROWS
	  CALL USRINP(ISIZE,4,'I',4,1,'BOUNDARIES=',
	1 'Enter start-column end-column start-row end-row:  (default '//
	2 '= entire map)')
C  Adjust the boundaries for correct reading from the map
	  CALL USRINP(SKY_PARAMETERS,2,'F',4,0,'SKY_PARAMETERS=',
	1 'Enter the sky and the sigma of the sky:')
	  CALL MESSAGE('Enter the multiple of sigma below which to reject '//
	1 'objects:')
	  K_SIGMA=2.0
	  CALL USRINP(K_SIGMA,1,'F',4,1,'MULTIPLE=','(default = 2)')
	  OBJDIAM=4
	  CALL USRINP(OBJDIAM,1,'I',4,1,'DIAMETER=','Enter the diameter of '//
	1 'the object in pixels:  (default = 4)')
C  Allocate some dynamic memory
	  NII=ISIZE(2)-ISIZE(1)+1
	  NJJ=ISIZE(4)-ISIZE(3)+1
	  LENGTH=NII*NJJ
	  CALL DYNAMIC(ADDRESS,LENGTH,'  R4',.TRUE.)
	  CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.TRUE.)
	  CALL FIND1(LU_IN,%VAL(ADDRESS),%VAL(ADDRESS2),NII,NJJ,ROWARRAY,
	1 ISIZE,SKY_PARAMETERS,K_SIGMA,OBJDIAM)
	  CALL DYNAMIC(ADDRESS,LENGTH,'  R4',.FALSE.)
	  CALL DYNAMIC(ADDRESS2,LENGTH,'  R4',.FALSE.)
	ELSE IF (COMMAND.EQ.'M') THEN
	  NUMBER_STARS=100
	  CALL USRINP(NUMBER_STARS,1,'I',4,1,'NUMBER_OF_STARS=',
	1 'Enter the number of stars desired in the file:  (default = 100)')
	  SKY=500.0
	  CALL USRINP(SKY,1,'F',4,1,'SKY=','Enter the desired sky value: '//
	1 '(default = 500)')
	  CALL USRINP(ADU_RANGE,2,'F',4,1,'ADU_RANGE=',
	1 'Enter the lowest and highest ADU values for the stars:')
	  CALL USRINP(FWHM,2,'F',4,0,'FWHM=','Enter the FWHM for the '//
	1 'major and minor axes:')
	  FWHMA=FWHM(1)
	  FWHMB=FWHM(2)
	  ECCENTRICITY=SQRT(1.0-(FWHMB/FWHMA)**2)
	  INCLINATION=0.0
	  CALL USRINP(INCLINATION,1,'F',4,1,'INCLINATION=',
	1 'Enter the inclination in degrees:  (default = 0)')
	  CALL MESSAGE('Enter the start-column end-column start-row end-row:')
	  CALL USRINP(BOUNDARIES,4,'F',4,0,'BOUNDARIES=',
	1 '(fractional values allowed)')
	  ISEED=12345
	  CALL USRINP(ISEED,1,'I',4,1,'SEED=','Enter a seed number:  '//
	1 '(default = 12345)')
	  XDELTA=BOUNDARIES(2)-BOUNDARIES(1)
	  YDELTA=BOUNDARIES(4)-BOUNDARIES(3)
	  ADUDELTA=ADU_RANGE(2)-ADU_RANGE(1)
	  DO I=1,NUMBER_STARS
	    ICOUNT=ICOUNT+1
	    IC=INT(LOG10(REAL(ICOUNT)))+1
	    ENCODE(6,2000,ID) ICOUNT
	    XMN=BOUNDARIES(1)+RAN(ISEED)*XDELTA
	    YMN=BOUNDARIES(3)+RAN(ISEED)*YDELTA
	    DATA=SKY+ADU_RANGE(1)+RAN(ISEED)*ADUDELTA
	    WRITE(21,1201) ID,XMN,YMN,INCLINATION,ECCENTRICITY,
	1   FWHMA,FWHMB,(IQ(JJ),JJ=1,4),DATA,SKY
	  END DO
	END IF
 900	CALL CLOSEFITS(LU_IN)
	CLOSE (UNIT=21)
C  If SAOimage was used, ask about erasing the region circles
	IF (COMMAND.EQ.'AS'.OR.COMMAND.EQ.'S') THEN
	  ERASE=.TRUE.
	  CALL USRINP(ERASE,1,'L',1,1,'ERASE=','Do you wish to erase the '//
	1 'circles?  (default = Y)')
	  IF (ERASE) CALL SAO_ERASE_CURSOR(1)
	END IF
	RETURN
	END
