	SUBROUTINE FITSMASK
C
C       This subroutine takes one FITS file and uses it as a mask
C  for another.  At every pixel location that the first is equal
C  to zero, the second is set equal to zero.  At every pixel
C  location where the value is greater than zero, the original 
C  value is retained.
C			Donald H. Gudehus, July, 1983
C			Version 2, D. H. Gudehus, May 12, 2008
C			Modified code for FITS and Unix.
C
	INCLUDE 'miipspar.for'
	INCLUDE 'fitsfacts.for'
C
	CHARACTER*3 PRGFMAT_IN,PRGFMAT_MASK
	CHARACTER*80 FILENAME_IN
C
	INTEGER PLANEKEY,STATUS
	INTEGER MAPS_IN(MAPMAX)
C
	LOGICAL IOFL,SP_IN,DP_IN,SP_MASK,DP_MASK
	
	REAL*4 MASKDATA(MAXSIZE),READDATA(MAXSIZE),WRITEDATA(MAXSIZE)
C
	REAL*8 MASKDATADP(MAXSIZE),READDATADP(MAXSIZE),WRITEDATADP(MAXSIZE)
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	CHARACTER*80 FILENAME
	LOGICAL EXTENDED
	COMMON /DFILE/ FILENAME,EXTENDED
C
	CALL MESSAGE('FITSMASK, Version 2')
	CALL MESSAGE('Open the FITS file to be operated on')
	SP_IN=.FALSE.
	DP_IN=.FALSE.
	LU_IN=11
	IOFL=.TRUE.
	FILENAME_IN=' '
	MAPKEY=0
	PLANEKEY=0
	NCOLS_IN=0
	NROWS_IN=0
	PRGFMAT_IN=' '
	CALL OPENFITS(LU_IN,IOFL,'FILE_IN=',FILENAME_IN,MAPKEY,PLANEKEY,NMAPS,
	1NPLNS,NCOLS_IN,NROWS_IN,PRGFMAT_IN,STATUS)
	IMNUM_IN=IMNUM
	IPNUM_IN=IPNUM
	IF (PRGFMAT_IN.EQ.'R8'.OR.PRGFMAT_IN.EQ.'C16') THEN
	  DP_IN=.TRUE.
	ELSE
	  SP_IN=.TRUE.
	END IF
C  Create a temporary file.  This allows for several options when we save our
C  results.
	LU_TEMP=12
	MAPS_IN(1)=IMNUM
	NMAPS_IN=1 !  Only 1 map is used here
	CALL OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS_IN,NROWS_IN,STATUS)
C  Open the mask file
	SP_MASK=.FALSE.
	DP_MASK=.FALSE.
	LU_MASK=13
	IOFL=.TRUE.
	FILENAME_MASK=' '
	MAPKEY=0
	PLANEKEY=0
	NCOLS_MASK=0
	NROWS_MASK=0
	PRGFMAT_MASK=' '
	CALL MESSAGE('Open the FITS file to be used as a mask')
	CALL OPENFITS(LU_MASK,IOFL,'FILE_MASK=',FILENAME_MASK,MAPKEY,
	1PLANEKEY,NMAPS,NPLNS,NCOLS,NROWS,PRGFMAT_MASK,STATUS)
	IMNUM_MASK=IMNUM
	IPNUM_MASK=IPNUM
	IF (PRGFMAT_MASK.EQ.'R8'.OR.PRGFMAT_MASK.EQ.'C16') THEN
	  DP_MASK=.TRUE.
	ELSE
	  SP_MASK=.TRUE.
	END IF
C  Loop through the rows
	DO J=1,NROWS_IN
	  IF (SP_IN) THEN
	    CALL RDFITSROW(LU_IN,IMNUM_IN,IPNUM_IN,READDATA,MAXSIZE,J)
	  ELSE IF (DP_IN) THEN
	    CALL RDDPFITSROW(LU_IN,IMNUM_IN,IPNUM_IN,READDATADP,MAXSIZE,J)
	  END IF
	  IF (SP_MASK) THEN
	    CALL RDFITSROW(LU_MASK,IMNUM_MASK,IPNUM_MASK,MASKDATA,MAXSIZE,J)
	  ELSE IF (DP_MASK) THEN
	    CALL RDDPFITSROW(LU_MASK,IMNUM_MASK,IPNUM_MASK,MASKDATADP,
	1   MAXSIZE,J)
	  END IF
	  IF (SP_IN) THEN
	    DO I=1,NCOLS_IN
C  Loop through the columns
	      IF ((SP_MASK.AND.MASKDATA(I).NE.0.0).OR.
	1     DP_MASK.AND.MASKDATADP(I).NE.0.0D0) THEN
	        WRITEDATA(I)=READDATA(I)
	      ELSE
	        WRITEDATA(I)=0.0
	      END IF
	    END DO
	    CALL WRFITSROW(LU_TEMP,IMNUM_IN,IPNUM_IN,WRITEDATA,MAXSIZE,J)
	  ELSE IF (DP_IN) THEN
	    DO I=1,NCOLS_IN
	      IF ((SP_MASK.AND.MASKDATA(I).NE.0.0).OR.
	1     DP_MASK.AND.MASKDATADP(I).NE.0.0D0) THEN
	        WRITEDATADP(I)=READDATADP(I)
	      ELSE
	        WRITEDATADP(I)=0.0D0
	      END IF
	    END DO
	    CALL WRFITSROW(LU_TEMP,IMNUM_IN,IPNUM_IN,WRITEDATA,MAXSIZE,J)
	  END IF
	END DO
	LU_OUT=14
	CALL SAVE_SAVE_AS(LU_IN,LU_TEMP,LU_OUT,MAPS_IN,NMAPS_IN,STATUS)
	CALL CLOSEFITS(LU_MASK)
	RETURN
	END
