	SUBROUTINE FLATSPEC
C
C       This subroutine fits polynomials to continuum segments between
C  mean X, Y points in spectral rows.  The spectra are then flattened 
C  relative to the interpolated or fitted continuum.
C		A. J. Pickles, August 6, 1981
C		VERSION 2, B. C. C., 9/10/81
C		ADDED VERSION, SCRATCH_BLOCK, & _SHARE ENTRY
C		Version 3, Donald H. Gudehus, March, l983
C		Added CHAR_SCRATCH_BLOCK
C		Version 4, Donald H. Gudehus, September 11, 2000
C		Introduced calls to OPENMS for output file, GETMAPS,
C		and AVERSIG (replacement for STATS1).  Renamed some variables.
C		Version 5, Donald H. Gudehus, September 9 to 18, 2004
C		Added graphical input of continuum values.  Replaced
C		call to CFIT and linear interpolation with call to
C		POLINTERP1D and polynomial interpolation.  Added polynomial
C		fitting.
C		Version 6, Donald H. Gudehus, May 16, 2008
C		Converted to FITS and Unix. For the initial plot, the
C		average of up to five of the highest value rows are used.
C		When several maps are input, the number of columns and rows
C		are limited to the smallest map in the set.
C
	include 'miipspar.for'
	include 'fitsfacts.for'
C
	PARAMETER (MAXBANDS=60,MAX_RANKS=5,
	1MAXNUMPARAM=20,DIMENSN_IND=1,DIMENSN_DEP=1)
C
	CHARACTER*1 INPUT_METHOD,INPUT_UNITS
	CHARACTER*2 METHOD
	CHARACTER*50 BIN,NEWBIN,CHFITSMHV,XLABEL,YLABEL,TITLE
	CHARACTER*80 FILENAME_IN
C
	INTEGER MAPS_IN(MAPMAX),ROWS_PROCESSED(MAXSIZE),
	1NCOLS_OUT(MAPMAX),NROWS_OUT(MAPMAX),STATUS
	INTEGER CHAN(2),BOUNDARIES(4),SIZE,COLOR(2)
	INTEGER NPLPOINTS(2),LINVECT(2),INC(2),LINTYPE(2),NSYMBOL(2),
	1IFOUND(MAXSIZE+4),DEGREE_INTERP,DEGREE_POLY
C
	LOGICAL COLUMNS_DIFFER,ROWS_DIFFER,INVERT(2),
	1CONSTANTTERMS(MAXNUMPARAM),INTERACT,CROSSHAIRS,THROUGH
C
	REAL*4 ROW_ARRAY(MAXSIZE),CONTIN(MAXSIZE),BANDS(2,MAXBANDS)
	REAL*4 CONT_IN_X(MAXBANDS),CONT_IN_Y(MAXBANDS)
	REAL*4 XPLOT(MAXSIZE+4,2),YPLOT(MAXSIZE+4,2)
C
	REAL*8 SUM,X(MAXSIZE),HIGHEST_VALUES(MAX_RANKS,3),DATA(MAXSIZE),
	1WEIGHT(MAXSIZE),COEFF(MAXNUMPARAM),
	2COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),MEANERROR,VALOUT,STDEV
C
	DATA BOUNDARIES/1,0,1,1/
C
	CALL MESSAGE ('FLATSPEC, Version 6')
	LU_IN=11
	FILENAME_IN=' '
	CALL MESSAGE('Open the file to have spectra flattened')
	CALL GET_INPUT_FITS_MAPS(LU_IN,FILENAME_IN,MAPS_IN,NMAPS_IN,STATUS)
	IF (STATUS.NE.0) THEN
	  CALL CLOSEFITS(LU_IN)
	  RETURN
	END IF
C  Create a temporary file
	LU_TEMP=12
	DO I=1,NMAPS_IN
	  IM=MAPS_IN(I)
	  NCOLS_OUT(I)=NAXIS1(IM,LU_IN)
	  NROWS_OUT(I)=NAXIS2(IM,LU_IN)
	END DO
	CALL OPEN_TEMP_FITS(LU_IN,LU_TEMP,FILENAME_IN,MAPS_IN,NMAPS_IN,
	1NCOLS_OUT,NROWS_OUT,STATUS)
C  Find the minimum number of rows in the input maps
	CALL GET_FITS_MAP_LIMITS(LU_IN,NCOLS_MIN,NCOLS_MAX,COLUMNS_DIFFER,
	1NROWS_MIN,NROWS_MAX,ROWS_DIFFER)
C  Get the number of rows to process
	DO J=1,NROWS_MIN
	  ROWS_PROCESSED(J)=J
	END DO
   5	IF (NROWS_MIN.GT.1) THEN
	  NROWS_PROCESSED=NROWS_MIN
	  CALL USRINP(ROWS_PROCESSED,NROWS_PROCESSED,'I',4,3,'ROWS=',
	1 'Enter the row numbers to be processed:  '//
	1 '(default = all based on smallest map)')
	  NROWS_PROCESSED=INEL(ROWS_PROCESSED,NROWS_MIN)
	ELSE
	  ROWS_PROCESSED(1)=1
	  NROWS_PROCESSED=1
	END IF
C  Check limits
	DO I=1,NROWS_PROCESSED
	  IF (ROWS_PROCESSED(I).GT.NROWS_MIN) THEN
	    CALL ERROR(-1,'A ROW NUMBER IS TOO LARGE')
	    GO TO 5
	  END IF
	END DO
C
	CALL MESSAGE('Enter the method to use for estimating the continuum '//
	1'values:')
	METHOD=' '
	DO WHILE (METHOD.NE.'PF'.AND.METHOD.NE.'PI')
	  METHOD='PI'
	  CALL MESSAGE('           PF = Polynomial fit')
	  CALL USRINP(%REF(METHOD),1,'A',2,1,'METHOD=',
	1 '           PI = Polynomial interpolation (default)')
	END DO
	IF (METHOD.EQ.'PF') THEN
	  DEGREE_POLY=6
	  CALL USRINP(DEGREE_POLY,1,'I',4,1,'DEGREE_FIT=','Enter the degree '//
	1 'the polynomial for the continuum fit:  (default = 6)')
	ELSE IF (METHOD.EQ.'PI') THEN
	  DEGREE_INTERP=3
	  CALL USRINP(DEGREE_INTERP,1,'I',4,1,'DEGREE_INTERPOLATION=',
	1 'Enter the degree of the polynomial for continuum interpolation:  '//
	2 '(default = 3)')
	END IF
C
	VAL=0.0
	CALL MESSAGE('Enter the value that the flattened continuum should '//
	1'be set to:')
	CALL USRINP(VAL,1,'F',4,1,'VALUE=','(default = average of input '//
	1'rows between lowest and highest columns)')
C
	INPUT_METHOD=' '
	DO WHILE (INPUT_METHOD.NE.'G'.AND.INPUT_METHOD.NE.'M')
	  INPUT_METHOD='G'
	  CALL MESSAGE('Enter the input method:')
	  CALL MESSAGE('         G = Graphics Device (default)')
	  CALL MESSAGE('         M = Manual Input')
	  CALL USRINP(%REF(INPUT_METHOD),1,'A',1,1,'INPUT_METHOD=',' ')
	END DO
	IF (INPUT_METHOD.EQ.'G') THEN
C  Plot the average of up to five rows having the highest intensities.  For 
C  simplicity, only look at the first plane in a map.
	  IP=1
	  NRANKS=MIN(MAX_RANKS,NROWS_PROCESSED)
	  DO I=1,NRANKS
	    HIGHEST_VALUES(I,1)=0.0D0
	  END DO
	  DO NM=1,NMAPS_IN
	    DO NR=1,NROWS_PROCESSED
	      SUM=0.0D0
	      CALL RDFITSROW(LU_IN,MAPS_IN(NM),IP,ROW_ARRAY,NCOLS_MAX,
	1     ROWS_PROCESSED(NR))
              DO NC=1,NCOLS_MIN
	        SUM=SUM+ROW_ARRAY(NC)
	      END DO
	      SUM=SUM/NCOLS_MIN
	      DO NV=1,NRANKS
C  HIGHEST_VALUES:  1st index is the rank, with 1 being the highest, 2nd
C  index contains the value, the map, and the row.
	        IF (SUM.GT.HIGHEST_VALUES(NV,1)) THEN
C  Shuffle down
	          DO IS=NRANKS,NV+1,-1
	            HIGHEST_VALUES(IS,1)=HIGHEST_VALUES(IS-1,1)
	          END DO
	          HIGHEST_VALUES(NV,1)=SUM
	          HIGHEST_VALUES(NV,2)=NM
	          HIGHEST_VALUES(NV,3)=NR
	          GO TO 10
	        END IF
	      END DO
  10	      CONTINUE
	    END DO
	  END DO
	  DO I=1,NCOLS_MIN
	    SUM=0.0D0
	    DO NV=1,NRANKS
	      CALL RDFITSROW(LU_IN,INT(HIGHEST_VALUES(NV,2)),IP,ROW_ARRAY,
	1     NCOLS_MAX,INT(HIGHEST_VALUES(NV,3)))
	      SUM=SUM+ROW_ARRAY(I)
	    END DO
	    YPLOT(I,1)=SUM/NRANKS
	  END DO
C  Set up the plotting parameters
	  SIZE=MAXSIZE+4	  
	  NPLPOINTS(1)=NCOLS_MIN
C  Initialize the plot arrays
	  DO I=NPLPOINTS(1)+1,SIZE
	    XPLOT(I,1)=0.0
	    YPLOT(I,1)=0.0
	  END DO
C  Assume first selected map
	  CALL BINVALFITS(LU_IN,MAPS_IN(1),BIN,XST,XND,XINC,.TRUE.,1)
	  IF (BIN.EQ.'WAVELENGTH'.OR.BIN.EQ.'WAVELENGTH(A)'.OR.
	1 BIN.EQ.'WAVEL(A)') THEN
	    DO I=1,NPLPOINTS(1)
	      XPLOT(I,1)=XST+XINC*(I-1+0.5)
	    END DO
	  ELSE
	    DO I=1,NCOLS_MIN
	      XPLOT(I,1)=I+0.5
	    END DO
	  END IF
	  NPLOTS=1
	  LINVECT(1)=0
	  LINVECT(2)=3
	  INC(1)=1
	  INC(2)=1
	  LINTYPE(1)=0
	  LINTYPE(2)=0
	  NSYMBOL(1)=0
	  NSYMBOL(2)=0
	  INVERT(1)=.FALSE.
	  INVERT(2)=.FALSE.
	  COLOR(1)=16776960
	  COLOR(2)=16711935
	  INTERACT=.FALSE.
	  CROSSHAIRS=.TRUE.
	  IF (BIN.EQ.'WAVELENGTH'.OR.BIN.EQ.'WAVELENGTH(A)'.OR.
	1 BIN.EQ.'WAVEL(A)') THEN
	    XLABEL='WAVELENGTH (Angstroms)'
	  ELSE
	    XLABEL='COLUMN'
	  END IF
	  YLABEL=CHFITSMHV(LU_IN,MAPS_IN(1),'BUNIT','ADU')
	  TITLE=CHFITSMHV(LU_IN,MAPS_IN(1),'OBJECT','UNDEF')
	  IF (TITLE.EQ.'UNDEF') THEN
	    TITLE=CHFITSMHV(LU_IN,MAPS_IN(1),'COMMENT','UNDEF')
	    IF (TITLE.EQ.'UNDEF'.OR.TITLE.EQ.'         ') THEN
	      TITLE=CHFITSMHV(LU_IN,MAPS_IN(1),'RUN',' ')
	    END IF
	  END IF
	  CALL MESSAGE(' ')
	  CALL MESSAGE('After the plot is drawn, press C to turn on the'//
	1 ' cursor.  Then position the')
	  CALL MESSAGE('cursor at the left end of the first band (low'//
	1 ' wavelength end) of continuum')
	  CALL MESSAGE('wavelengths to be fit and press the <CR>.  Then'//
	1 ' position the cursor at the')
	  CALL MESSAGE('right end of the first band and press the <CR>.'//
	1 '  Repeat these steps for each')
	  CALL MESSAGE('additional continuum band to be used in the fit.'//
	1 '  Mistakes can be voided by')
	  CALL MESSAGE('pressing the - key.  Press Q to turn off the cursor'//
	1 ' and <CR> to proceed.')
	  CALL MESSAGE(' ')
C	  THROUGH=.TRUE.
C	  CALL USRINP(THROUGH,1,'L',4,3,'THROUGH=','Press the <CR> when'//
C	1 ' through reading these instructions')
	  CALL XYPLOT(XPLOT,YPLOT,SIZE,NPLOTS,NPLPOINTS,LINVECT,INC,
	1 LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,YLABEL,
	2 TITLE,' ',.FALSE.)
	  CALL RESETTEK
C  The points selected with the cursor are placed in XCOORD and YCOORD
C  by XYPLOT.
C  
C  Search XPLOT and YPLOT for the points closest to where the cursor was
C  positioned
	  CALL FINDCOORD(XPLOT,YPLOT,IFOUND,SIZE-4,NPOINTSCURSOR,2)
C  IFOUND will contain the indexes of the points closest to the cursor
	  NBANDS=(NPOINTSCURSOR)/2
	  WRITE(6,1000) NBANDS
1000	  FORMAT(' ',I3,' bands were chosen between the limits of')
	  WRITE(7,1000) NBANDS
	  WRITE(6,1001) XPLOT(IFOUND(1),1),XPLOT(IFOUND(NPOINTSCURSOR),1)
1001	  FORMAT(' ',G12.6,' and ',G12.6,'.')
	  WRITE(7,1001) XPLOT(IFOUND(1),1),XPLOT(IFOUND(NPOINTSCURSOR),1)
	  DO I=1,NBANDS
	    BANDS(1,I)=IFOUND(I*2-1)
	    BANDS(2,I)=IFOUND(I*2)
	  END DO
C  Set the boundaries for computing the mean 
	  BOUNDARIES(1)=IFOUND(1)
	  BOUNDARIES(2)=IFOUND(NPOINTSCURSOR)
	ELSE IF (INPUT_METHOD.EQ.'M') THEN
C  For simplicity, only look at the first plane in a map.
	  IP=1
	  IF (VAL.EQ.0.) THEN
	    BOUNDARIES(2)=NCOLS_MIN
	    CALL MESSAGE('Enter the starting and ending columns for the '//
	1   'region over which the')
	    CALL USRINP(BOUNDARIES,2,'I',4,1,'COLUMNS=',
	1   'continuum mean is to be calculated:  (default = all)')
	  END IF
	  CALL BINVALFITS(LU_IN,MAPS_IN(1),BIN,XST,XND,XINC,.TRUE.,1)
	  WRITE(6,2000) BIN,XST,XEND,XINC
	  WRITE(7,2000) BIN,XST,XEND,XINC
2000	  FORMAT(' Bin type is 'A9/' Start, end, and incremental values '//
	1 'are:'/2X,3(1PG12.5)/)
C
	  INPUT_UNITS=' '
	  DO WHILE (INPUT_UNITS.NE.'B'.AND.INPUT_UNITS.NE.'C')
	    INPUT_UNITS='B'
	    CALL MESSAGE('Enter the input method:')
	    CALL MESSAGE('         B = Band values in bin type (default)')
	    CALL MESSAGE('         C = Band values in columns')
	    CALL USRINP(%REF(INPUT_UNITS),1,'A',1,1,'INPUT_UNITS=',' ')
	  END DO
	  CALL MESSAGE('Enter the starting and ending values of each '//
	1 'desired continuum band.')
	  NBANDS=MAXBANDS
	  CALL ZERO(BANDS,MAXBANDS*2)
	  DO N=1,MAXBANDS
  15	    CALL USRINP(BANDS(1,N),2,'F',4,3,'BANDS=',
	1   'Enter only two values for each band;  enter <CR> to proceed:')
	    NB=NEL(BANDS(1,N),2)
	    IF (NB.LT.2) THEN
	      CALL ERROR(-1,'You must enter at least two values;  try again')
	      GO TO 15
	    ELSE IF (BANDS(2,N).LT.BANDS(1,N)) THEN
	      CALL ERROR(-1,'The second value of each pair must be '//
	1     'numerically higher than the first.')
	      GO TO 15
	    END IF
	    IF (N.GT.2) THEN
	      IF (BANDS(1,N).EQ.0.0.AND.BANDS(2,N).EQ.0.0) THEN
	        NBANDS=N-1
	        GO TO 20
	      END IF
	    END IF
	  END DO
  20	  CONTINUE
	END IF
C  Initialize the weights
	DO I=1,NCOLS_MIN
	  WEIGHT(I)=1.0D0
C  Set up the array for returning values from LINFIT1D
	  XPLOT(I,2)=I
	END DO
C
C  Start of the loop over the input maps in which each selected
C  row is flattened
C
	DO 100 M=1,NMAPS_IN
	  CALL BINVALFITS(LU_IN,MAPS_IN(M),NEWBIN,XST,XND,XINC,.TRUE.,1)
	  WRITE(6,2100) MAPS_IN(M),NEWBIN
	  WRITE(7,2100) MAPS_IN(M),NEWBIN
2100	  FORMAT(' Map = 'I4'  Bintype = 'A9)
	  IF (INPUT_UNITS.EQ.'B'.AND.NEWBIN.NE.BIN) THEN
	    CALL ERROR(-1,'Incorrect Bintype')
	  END IF
C  Always copy the map header
c	  CALL COPMHD(11,MAPS_IN(M),12,MAPS_OUT(M))
C  Compute the average of all the input rows between the lowest and
C  highest columns.
	  IF (VAL.EQ.0.) THEN !  For VAL = 0, compute the average
	    SUM=0.0D0
	    DO J=1,NROWS_PROCESSED
	      CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,ROW_ARRAY,NCOLS_MAX,
	1     ROWS_PROCESSED(J))
	      CALL AVERSIG(ROW_ARRAY,NCOLS_MIN,1,BOUNDARIES(1),
	1     1,BOUNDARIES(2),1,.FALSE.,
	2     VALOUT,STDEV)
	      SUM=SUM+VALOUT
	    END DO
	    VALOUT=SUM/NROWS_PROCESSED
	  ELSE  !  For VAL not = 0, use the value entered
	    VALOUT=VAL
	  END IF
	  DO J=1,NROWS_PROCESSED
	    CALL RDFITSROW(LU_IN,MAPS_IN(M),IP,ROW_ARRAY,NCOLS_MAX,
	1   ROWS_PROCESSED(J))
	    NC=0 !  The current band number
	    DO N=1,NBANDS
	      IF (INPUT_UNITS.EQ.'B') THEN
	        DO I=1,2
	          CHAN(I)=NINT((BANDS(I,N)-XST)/XINC)+1
	        END DO
	      ELSE
	        DO I=1,2
	          CHAN(I)=BANDS(I,N)
	        END DO
	      END IF
C  Compute the average within a band
	      IF (CHAN(1).GE.1.AND.CHAN(2).LE.NCOLS_MIN
	1     .AND.CHAN(2).GE.CHAN(1)) THEN
	        SUM=0.0D0
	        NC=NC+1
	        DO I=CHAN(1),CHAN(2)
	          SUM=SUM+ROW_ARRAY(I)
	        END DO
	        CONT_IN_Y(NC)=SUM/(CHAN(2)-CHAN(1)+1)
	        CONT_IN_X(NC)=(CHAN(1)+CHAN(2))/2.0
	      ELSE
	        WRITE(6,3000) N
	        WRITE(7,3000) N
3000	        FORMAT(' Band 'I4)
	         CALL ERROR(-1,
	1        'Column specification out of range - not used')
	      END IF
	    END DO !  Ends loop over bands
	    IF ((METHOD.EQ.'PF'.AND.NC.GE.DEGREE_POLY+1).OR.
	1   (METHOD.EQ.'PI'.AND.NC.GE.DEGREE_INTERP+1)) THEN
	      IF (METHOD.EQ.'PF') THEN
	        DO I=1,NC
	          X(I)=CONT_IN_X(I)
	          DATA(I)=CONT_IN_Y(I)
	        END DO
	        CALL LINFIT1D(X,DATA,WEIGHT,NC,DEGREE_POLY,CONSTANTTERMS,
	1       COEFF,COVARIANCE,MAXNUMPARAM,MEANERROR,XPLOT(1,2),CONTIN,
	2       NCOLS_MIN)
	        WRITE(6,3500) MEANERROR
3500	        FORMAT(' ','The mean error of the polynomial fit is ',
	1       G12.6)
	        WRITE(7,3500)
	      ELSE IF (METHOD.EQ.'PI') THEN
	        DO I=1,NCOLS_MIN
	          CALL POLINTERP1D(CONT_IN_X,CONT_IN_Y,NC,I*1.0,CONTIN(I),
	1         DEGREE_INTERP,0,COEFF)
	        END DO
	      END IF
	    ELSE
	      CALL ERROR(-1,'Insufficient continuum points, no flattening '//
	1     'produced')
	      GO TO 100
	    END IF
C  Flatten
	    DO I=1,NCOLS_MIN
	      IF (CONTIN(I).EQ.0.0) CONTIN(I)=1.0
	      ROW_ARRAY(I)=VALOUT*ROW_ARRAY(I)/CONTIN(I)
	    END DO
	    CALL WRFITSROW(LU_TEMP,MAPS_IN(M),IP,ROW_ARRAY,NCOLS_MAX,
	1   ROWS_PROCESSED(J))
	  END DO !  Ends loop over rows
 100	CONTINUE !  Ends loop over maps
C
c call save..
	LU_OUT=13
	CALL SAVE_SAVE_AS(LU_IN,LU_TEMP,LU_OUT,MAPS_IN,NMAPS_IN,STATUS)
	RETURN
	END
