	SUBROUTINE ANNULUS1(LU_IN,DAT,NCOL,NROW,RADIUS,ADU)
C
C       This subroutine helps ANNULUS and allows operation of DYNAMIC.
C  The calculated values are passed back to ANNULUS because writing
C  to a character file before calling DYNAMIC to decrease the virtual
C  memory will cause an error.
C			Donald H. Gudehus, August, l982
C			Version 2, Donald H. Gudehus, January 31, 2008
C			Revised argument list
C			Version 2U, Donald H. Gudehus, January 31, 2008
C			Converted to Unix and FITS
C
C	LU_IN	(C)	The logical unit of the file to be read
C	DAT	(C)	A temporary array used to read from an XY area
C	NCOL	(C)	The number of columns in the area
C	NROW	(C)	The number of rows in the area
C	RADIUS	(R)	An array containing the radii
C	ADU	(R)	An array containing the average intensity along
C			an annulus.

C
	INCLUDE 'miipspar.for'
C
	REAL*4 INTERP2D,LOGADU
	REAL*4 DAT(NCOL,NROW)
	REAL*4 RADIUS(MAXDIAG),ADU(MAXDIAG)
C
	INTEGER ADDRESS,LENGTH
	LOGICAL MORE,OFIL,MESSAG
C
	INTEGER RADII(2),NCOLS,NROWS
	REAL*4 COLUMN,ROW,POSITION(2),INCLINA
c	COMMON /INTEGER4_SCRATCH_BLOCK2/ RADII,ADDRESS,LENGTH,MORE,OFIL,MESSAG,
c	1COLUMN,ROW,POSITION,INCLINA,NI,NJ,MNCOLUMN,MNROW,MXCOLUMN,MXROW,
c	2ECCENTR,FORESHORT,ADUMIN,SKY,PI,CS,SN,A,B,DELTAE,
c	3PX,NPX,X,Y,ESTART,ANGLIMIT,E
	COMMON /INTEGER4_SCRATCH_BLOCK2/ RADII,NCOLS,NROWS,
	1MNCOLUMN,MNROW,MXCOLUMN,MXROW
	COMMON /REAL4_SCRATCH_BLOCK2/ COLUMN,ROW,INCLINA,
	1ECCENTR,FORESHORT
C
	CHARACTER*6 ID !  Variable for object name
	COMMON /ID/ID
C
	COMMON /MAPN/ IMNUM
	COMMON /PLANEN/ IPNUM
C
	PI=3.141592654
C  Read from the map
	CALL RDFITSXY(LU_IN,IMNUM,IPNUM,DAT,NCOL,NROW,
	1MNCOLUMN,NCOL,MNROW,NROW)
C  Annulus calculations
        CS=COS(INCLINA/57.29577951)
	SN=SIN(INCLINA/57.29577951)
	FORESHORT=SQRT(1.0-ECCENTR**2)
C  Get the angular limits
	IF (COLUMN.LT.MXCOLUMN.AND.ROW.LT.MNROW+1) THEN !  Bottom and
							!  left
	  TANGENT=(MNROW+1-ROW)/(MXCOLUMN-COLUMN)
	  ESTART=ATAN(TANGENT)
	ELSE IF (COLUMN.GT.MXCOLUMN.AND.ROW.LT.MXROW) THEN !  Right and
							   !  bottom
	  TANGENT=(MXROW-ROW)/(MXCOLUMN-COLUMN)
	  ESTART=ATAN(TANGENT)+PI
	ELSE IF (COLUMN.GT.MNCOLUMN+1.AND.ROW.GE.MXROW) THEN !  Top
							     !  and
							     !  right
	  TANGENT=(MXROW-ROW)/(MNCOLUMN+1-COLUMN)
	  ESTART=ATAN(TANGENT)+PI
	ELSE IF (COLUMN.LT.MNCOLUMN+1.AND.ROW.GE.MNROW+1) THEN !  Left
							       !  and
							       !  top
	  TANGENT=(MNROW+1-ROW)/(MNCOLUMN+1-COLUMN)
	  ESTART=ATAN(TANGENT)+2*PI
	ELSE
	  ESTART=0.0  ! Center within region of data
	END IF
	IF (COLUMN.LT.MNCOLUMN+1.AND.ROW.LT.MXROW) THEN !  Left and
							!  bottom
	  TANGENT=(MXROW-ROW)/(MNCOLUMN+1-COLUMN)
	  ANGLIMIT=ATAN(TANGENT)
	ELSE IF (COLUMN.GT.MNCOLUMN+1.AND.ROW.LT.MNROW+1) THEN !  Bottom
							       !  and
							       !  right
	  TANGENT=(MNROW+1-ROW)/(MNCOLUMN+1-COLUMN)
	  ANGLIMIT=ATAN(TANGENT)+PI
	ELSE IF (COLUMN.GT.MXCOLUMN.AND.ROW.GE.MNROW+1) THEN !  Right
							     !  and
							     !  top
	  TANGENT=(MNROW+1-ROW)/(MXCOLUMN-COLUMN)
	  ANGLIMIT=ATAN(TANGENT)+PI
	ELSE IF (COLUMN.LT.MXCOLUMN.AND.ROW.GE.MXROW) THEN !  Top and
							   !  left
	  TANGENT=(MXROW-ROW)/(MXCOLUMN-COLUMN)
	  ANGLIMIT=ATAN(TANGENT)+2*PI
	ELSE
	  ANGLIMIT=6.28318
	END IF
	IF (ANGLIMIT-E.GT.6.28318) CALL ERROR(-1,'Difference of anguluar
	1 limits too large')
	DO  20 K=RADII(1),RADII(2)
	  A=K
	  B=A*FORESHORT
C  Calculate the angular increment
	  IF (A.GT.0.0) THEN
	    DELTAE=0.6/A
	  ELSE
	    DELTAE=6.28
	  END IF
	  PX=0.0
	  NPX=0
	  E=ESTART
C  Get X and Y for the annulus by means of a radius vector originating
C  from the center of the ellipse;  increment along the circumference
C  of a circle with radius equal to A in equal amounts, ie., 0.1 or
C  0.6 pixels.
	  DO 10 WHILE (E.LT.ANGLIMIT)
C  Increment by 1 because array index starts at 1
	    X=COLUMN-FLOATJ(MNCOLUMN)+1+A*COS(E)*CS-B*SIN(E)*SN
	    IF (NROWS.EQ.1.AND.E.EQ.ESTART) THEN
	      Y=1.5
	    ELSE
	      Y=ROW-FLOATJ(MNROW)+1+A*COS(E)*SN+B*SIN(E)*CS
	    END IF
	    E=E+DELTAE
	    PX1=INTERP2D(DAT,NCOL,NROW,X,Y)
	    IF (PX1.EQ.0.) GO TO 10
	    PX=PX+PX1
	    NPX=NPX+1
10	  CONTINUE
	  PX=PX/MAX0(NPX,1)
	  RADIUS(K-RADII(1)+1)=K
	  ADU(K-RADII(1)+1)=PX
 20     CONTINUE
	RETURN
	END
