      SUBROUTINE FWHM(DAT,NCOL,NROW,ARRAY,X,Y,THETA,FWHMA,FWHMB)
C
C       This subroutine calculates the full-width-half-maximum along
C  the major and minor axes of an elliptical distribution in two
C  dimensions.
C			Donald H. Gudehus, January, 1983
C
C	DAT	(C)	The array containing the pixel information
C	NCOL	(C)	The dimension of DAT in the X direction
C	NROW	(C)	The dimension of DAT in the Y direction
C	ARRAY	(R)	A working array
C	X	(C)	The location of the centroid of the object
C			under consideration relative to the beginning
C			of the array DAT.
C	Y	(C)	The same as X but in the Y direction 
C	THETA	(C)	The inclination angle measured in degrees
C	FWHMA	(R)	The full-width-half-maximum along the major
C			axis.
C	FWHMB	(R)	The full-width-half-maximum along the minor
C			axis.
C
	DIMENSION DAT(NCOL,NROW),ARRAY(1)
	REAL SINTERP2D
	REAL*8 ABCOS,ABSIN,ANGLE,XX,YY,XINDEX,XSTART,XEND,YINDEX,
	1YSTART,YEND
	LOGICAL MAJORAXIS
C
C  Make XINDEX and YINDEX be measured from 1.0
	XINDEX=X+1.0
	YINDEX=Y+1.0
	IF (XINDEX.LT.1.0.OR.XINDEX.GT.FLOAT(NCOL).OR.YINDEX.LT.1.0
	1.OR.YINDEX.GT.FLOAT(NROW)) THEN
	  FWHMA=0.0
	  FWHMB=0.0
	  CALL ERROR(-1,'Centroid too close to edge')
	  RETURN
	END IF
C  Find the minimum value in DAT
	ADUMIN=DAT(1,1)
	DO J=1,NROW
	  DO I=1,NCOL
	    IF (DAT(I,J).LT.ADUMIN) ADUMIN=DAT(I,J)
	  END DO
	END DO
	ADUHALF=(DAT(JIDINT(XINDEX),JIDINT(YINDEX))-ADUMIN)/2.0+ADUMIN
	LIMIT=MIN0(NCOL,NROW)
	ANGLE=THETA
	DO 200 N=1,2
C  Load ARRAY with the values along the major axis if N=1 and with the
C  values along the minor axis when N=2
	  IF (N.EQ.2) ANGLE=THETA+90.0
C  Make ANGLE be in the first or second quadrants
  45	  IF (ANGLE.LT.0.0) THEN
	    ANGLE=ANGLE+180.0
	    GO TO 45
	  END IF
  48	  IF (ANGLE.GT.180.0) THEN
	    ANGLE=ANGLE-180.0
	    GO TO 48
	  END IF
C  Determine which quadrant ANGLE actually is in
	  NQUAD=1
	  IF (ANGLE.GE.90.0) NQUAD=2
	  ANGLE=ANGLE/57.29577951
C  The limits below protect against reading outside the useful area of
C  DAT and from reading an unnecessarily large region.
	  ABCOS=DABS(DCOS(ANGLE))
	  ABSIN=DABS(DSIN(ANGLE))
	  LIMITW=LIMIT
  	  XSTART=XINDEX-DBLE(LIMITW/2)*ABCOS
  	  IF (XSTART.LT.1.0) XSTART=1.0
	  IF (NQUAD.EQ.1) THEN
	    YSTART=YINDEX-DBLE(LIMITW/2)*ABSIN
  	    IF (YSTART.LT.1.0) YSTART=1.0
	  ELSE IF (NQUAD.EQ.2) THEN
	    YSTART=YINDEX+DBLE(LIMITW/2)*ABSIN
  	    IF (YSTART.GT.FLOATJ(NCOL)) YSTART=NCOL
	  END IF
  	  XEND=XINDEX+DBLE(LIMITW/2)*ABCOS
  	  IF (XEND.GT.FLOATJ(NCOL)) XEND=NCOL
	  IF (NQUAD.EQ.1) THEN
	    YEND=YINDEX+DBLE(LIMITW/2)*ABSIN
  	    IF (YEND.GT.FLOATJ(NCOL)) YEND=NCOL
	  ELSE IF (NQUAD.EQ.2) THEN
	    YEND=YINDEX-DBLE(LIMITW/2)*ABSIN
  	    IF (YEND.LT.1.0) YEND=1.0
	  END IF
C  Initialize ARRAY
	  DO K=1,LIMIT
	    ARRAY(K)=0.0
	  END DO
	  DO 100 K=1,LIMIT
	    XX=XSTART+DBLE(K-1)*ABCOS
	    IF (NQUAD.EQ.1) THEN
	      YY=YSTART+DBLE(K-1)*ABSIN
	      IF (XX.GT.XEND.OR.YY.GT.YEND) GO TO 100
	    ELSE IF (NQUAD.EQ.2) THEN
	      YY=YSTART-DBLE(K-1)*ABSIN
	      IF (XX.GT.XEND.OR.YY.LT.YEND) GO TO 100
	    END IF
	    ARRAY(K)=SINTERP2D(DAT,NCOL,NROW,SNGL(XX),SNGL(YY),5,1.33)
 100	  CONTINUE
C  Right or upper side
	  DO 150 I=LIMIT,1,-1
	    IF (ARRAY(I).GT.ADUHALF) THEN
	      DRIGHT=(ADUHALF-ARRAY(I))/(ARRAY(I+1)-ARRAY(I))+
	1     FLOATJ(I)
	      GO TO 155
	    END IF
 150	  CONTINUE
C  Left or bottom side
 155	  DO 160 I=1,LIMIT
	    IF (ARRAY(I).GT.ADUHALF) THEN
	      DLEFT=(ADUHALF-ARRAY(I-1))/(ARRAY(I)-ARRAY(I-1))+FLOATJ(I)
	1     -1.0
	      GO TO 165
	    END IF
 160	  CONTINUE
 165	  IF (N.EQ.1) THEN
	    FWHMA=DRIGHT-DLEFT
	  ELSE
	    FWHMB=DRIGHT-DLEFT
	  END IF
 200	CONTINUE
	DO I=1,LIMIT
	END DO
	RETURN
	END
