	REAL*8 FUNCTION INTERPDP2D(DATADP,NCOL,NROW,X,Y)
C
C       This function does two-dimensional interpolation when given a 
C  fractional pixel location.  The pixel intensities of four pixels
C  are averaged with weights determined by the area of overlap of a
C  single pixel centered on the given fractional location.  The actual
C  pixels used are determined by what quadrant of the original pixel
C  the given point lies in.  This is the double precision version of
C  interp2d.
C				Donald H. Gudehus, February 25, 2008
C
C	DATADP	(C)	The array containing the data
C	NCOL	(C)	The dimension of DATADP in the x direction
C	NROW	(C)	The dimension of DATADP in the y direction
C	X	(C)	The pixel location in the x direction where
C			X=1.0 is the smallest value that can
C			reference the array.  For accuracy in interpolation,
C			X should not be smaller than 1.5.
C	Y	(C)	Like X but in the y direction
C
	REAL*8 DATADP(NCOL,NROW)
C 
	DEL=0.0000001
	INTERPDP2D=0.0D0
	IX=X
	IY=Y
C  Return a zero if the location is outside the range of the array
	IF (IX.LT.1.OR.IX.GE.NCOL+1) RETURN
	IF (IY.LT.1.OR.IY.GE.NROW+1) RETURN
C  Return a non-zero uninterpolated value if the location is within the half-
C  pixel guard area.
	IF (X.LT.1.5+DEL.OR.X.GT.FLOAT(NCOL)+0.5-DEL.OR.
	1Y.LT.1.5+DEL.OR.Y.GE.FLOAT(NROW)+0.5-DEL) THEN
	  INTERPDP2D=DATADP(IX,IY)
	  RETURN
	END IF
C  Special section for files which consist of one row.  Y should be equal
C  to 1.5 in the call.
	IF (NROW.EQ.1) THEN
	  IF (X-IX.GE.0.5) THEN
C  Don't interpolate into an edited area
	    IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX+1,IY).EQ.0.0D0) THEN
	      INTERPDP2D=DATADP(IX,IY)
	      RETURN
	    END IF
	    INTERPDP2D=DATADP(IX,IY)*(IX-X+1.5D0)+DATADP(IX+1,IY)*(X-IX-0.5D0)
	  ELSE IF (X-IX.LT.0.5) THEN
	    IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX-1,IY).EQ.0.0D0) THEN
	      INTERPDP2D=DATADP(IX,IY)
	      RETURN
	    END IF
	    INTERPDP2D=DATADP(IX,IY)*(X-IX+0.5D0)+DATADP(IX-1,IY)*(IX-X+0.5D0)
	  END IF
	END IF
C  First quadrant (includes center of pixel)
	IF (X-IX.GE.0.5.AND.Y-IY.GE.0.5) THEN
	  IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX+1,IY).EQ.0.0D0.OR.
	1 DATADP(IX+1,IY+1).EQ.0.0D0.OR.DATADP(IX,IY+1).EQ.0.0D0) THEN
	    INTERPDP2D=DATADP(IX,IY) !  Covers first case of center of pixel
	    RETURN
	  END IF
	  INTERPDP2D=DATADP(IX,IY)*(IX-X+1.5D0)*(IY-Y+1.5D0) !  Covers second
	1 +DATADP(IX+1,IY)*(X-IX-0.5D0)*(IY-Y+1.5D0) !  case of center of pixel
	1 +DATADP(IX+1,IY+1)*(X-IX-0.5D0)*(Y-IY-0.5D0)
	1 +DATADP(IX,IY+1)*(IX-X+1.5D0)*(Y-IY-0.5D0)
C  Second quadrant
	ELSE IF (X-IX.LT.0.5.AND.Y-IY.GE.0.5) THEN
	  IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX,IY+1).EQ.0.0D0.OR.
	1 DATADP(IX-1,IY+1).EQ.0.0D0.OR.DATADP(IX-1,IY).EQ.0.0D0) THEN
	    INTERPDP2D=DATADP(IX,IY)
	    RETURN
	  END IF
	  INTERPDP2D=DATADP(IX,IY)*(X-IX+0.5D0)*(IY-Y+1.5D0)
	1 +DATADP(IX,IY+1)*(X-IX+0.5D0)*(Y-IY-0.5D0)
	1 +DATADP(IX-1,IY+1)*(IX-X+0.5D0)*(Y-IY-0.5D0)
	1 +DATADP(IX-1,IY)*(IX-X+0.5D0)*(IY-Y+1.5D0)
C  Third quadrant
	ELSE IF (X-IX.LT.0.5.AND.Y-IY.LT.0.5) THEN
	  IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX-1,IY).EQ.0.0D0.OR.
	1 DATADP(IX-1,IY-1).EQ.0.0D0.OR.DATADP(IX,IY-1).EQ.0.0D0) THEN
	    INTERPDP2D=DATADP(IX,IY)
	    RETURN
	  END IF
	  INTERPDP2D=DATADP(IX,IY)*(X-IX+0.5D0)*(Y-IY+0.5D0)
	1 +DATADP(IX-1,IY)*(IX-X+0.5D0)*(Y-IY+0.5D0)
	1 +DATADP(IX-1,IY-1)*(IX-X+0.5D0)*(IY-Y+0.5D0)
	1 +DATADP(IX,IY-1)*(X-IX+0.5D0)*(IY-Y+0.5D0)
C  Fourth quadrant
	ELSE IF (X-IX.GE.0.5.AND.Y-IY.LT.0.5) THEN
	  IF (DATADP(IX,IY).EQ.0.0D0.OR.DATADP(IX,IY-1).EQ.0.0D0.OR.
	1 DATADP(IX+1,IY-1).EQ.0.0D0.OR.DATADP(IX+1,IY).EQ.0.0D0) THEN
	    INTERPDP2D=DATADP(IX,IY)
	    RETURN
	  END IF
	  INTERPDP2D=DATADP(IX,IY)*(IX-X+1.5D0)*(Y-IY+0.5D0)
	1 +DATADP(IX,IY-1)*(IX-X+1.5D0)*(IY-Y+0.5D0)
	1 +DATADP(IX+1,IY-1)*(X-IX-0.5D0)*(IY-Y+0.5D0)
	1 +DATADP(IX+1,IY)*(X-IX-0.5D0)*(Y-IY+0.5D0)
	ELSE
	  CALL MESSAGE('Error in INTERPDP2D')
	END IF
	RETURN
	END
