	SUBROUTINE LINFIT1D1(X,DATA,WEIGHT,NPOINTS,DEGREE,
	1CONSTANTTERMS,COEFF,COVARIANCE,MAXNUMPARAM,MEANERROR,
	2XPLOT,YPLOT,NPLPOINTS,NCONSTANT,ISHIFT,KPVT,DUMCOEFF,
	3LEFTSIDE,UPTRIPART,AUX)
C
C       This subroutine does a linear least squares fit to
C  weighted data and returns the parameters of the fit and their
C  errors.  If desired, certain coefficients may be held constant in
C  the solution.  There is no limit on the degree of the polynomial.
C  For plotting purposes, an array is filled with calculated values
C  corresponding to the values in the array XPLOT.  For spline fits,
C  this is the only meaningful returned value.  To prevent
C  a page owner violation when the user is prompted for the degree
C  (DEGREE=-1 in the call), the calling program should either contain a 
C  call to USRINP or open units 7 and 49.
C
C			Donald H. Gudehus, August, 1984
C			Version 2, Donald H. Gudehus, April 17, 2000
C			Used NUM_DATA instead on NPOINTS in calculation
C			of MEAN_ERROR.
C
C	X	      (C) An array containing the values of the
C			  independent variable
C	DATA	      (C) An array containing the observations to
C			  be fitted
C	WEIGHT	      (C) An array for the weights of each point
C	NPOINTS	      (C) The dimension of the above three arrays
C			  and the number of points to be fit
C	DEGREE	    (C,R) The degree of the polynomial fit if .GE. 0.
C			  If DEGREE=-1, the degree of the polynomial fit is
C			  asked for;  if DEGREE=-2, the degree of the
C			  polynomial fit and the coefficients which are to
C			  be held constant are asked for;  if DEGREE=-3,
C			  the type of fit is asked for and if it is to be a
C			  polynomial, the degree and coefficients which are
C			  to be held constant are asked for;  if DEGREE=-99,
C			  a spline fit is done.  
C	CONSTANTTERMS (C) An array which is .TRUE. if that coefficient
C			  is to be held constant and not solved for.
C	COEFF	    (C/R) An array containing the coefficients of the
C			  polynomial of the fit.
C	COVARIANCE    (R) An array containing the covariances
C	MAXNUMPARAM   (C) The maximum number of parameters to be used
C			  in the calling program and the dimensions of
C			  COEFF and COVARIANCE in the calling program.
C	MEANERROR     (R) The mean error for an observation of
C			  average weight
C	XPLOT	      (C) An array containing points to be evaluated
C			  with the fitted function
C	YPLOT	      (R) An array containing the evaluated points from
C			  XPLOT
C	NPLPOINTS     (C) The dimension of the above two arrays and the
C			  number of points contained therein.
C
	INTEGER DEGREE,DUMMYDEGREE,NCONSTANT(MAXNUMPARAM),
	1ISHIFT(MAXNUMPARAM),KPVT(MAXNUMPARAM)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM)
	REAL XPLOT(NPLPOINTS),YPLOT(NPLPOINTS)
	REAL*8 X(NPOINTS),DATA(NPOINTS),WEIGHT(NPOINTS),
	1COEFF(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR,
	3DUMCOEFF(MAXNUMPARAM),LEFTSIDE(MAXNUMPARAM,MAXNUMPARAM),
	4UPTRIPART(MAXNUMPARAM*(MAXNUMPARAM+1)/2),AUX(MAX(MAXNUMPARAM-1,1)),
	5SIGMASUM,SUMWEIGHTS,AVWEIGHT,TEMPX(3),TEMPDATA(3),TEMPWEIGHT(3),
	6POL
C
	CHARACTER*1 TYPE
C
	DUMMYDEGREE=DEGREE
	IF (DUMMYDEGREE.EQ.-99) THEN
	  TYPE='S'
	ELSE
	  TYPE='P'
	END IF
	IF (DUMMYDEGREE.EQ.-3) THEN
	  CALL MESSAGE('Enter the type of fit:')
	  CALL MESSAGE('     P = Polynomial (default)')
	  CALL MESSAGE('     S = Spline approximation')
	  CALL USRINP(%REF(TYPE),1,'A',1,1,'TYPE=',' ')
	  DUMMYDEGREE=-2
	END IF
	IF (TYPE.EQ.'P') THEN
	  IF (DUMMYDEGREE.EQ.-1.OR.DUMMYDEGREE.EQ.-2) THEN
	    DEGREE=1
	    CALL USRINP(DEGREE,1,'I',4,1,'DEGREE=',
	1   'Enter the degree of the fit:  (default = 1)')
	  END IF
	  IF (DUMMYDEGREE.EQ.-2) THEN
	    DO I=1,MAXNUMPARAM
	      J=SETEND(NCONSTANT(I))  ! Essential here if a default is taken
	    END DO
	    CALL MESSAGE('Enter the numbers of the parameters which are '//
	1   'to be held constant:')
	    CALL USRINP(NCONSTANT,MAXNUMPARAM,'I',4,3,'NUMBERS_CONSTANT=',
	1   '(default = none)')
	    NUMCONSTANT=NEL(NCONSTANT,MAXNUMPARAM)
	    DO I=1,MAXNUMPARAM
	      CONSTANTTERMS(I)=.FALSE.
	    END DO
	    DO I=1,NUMCONSTANT
	      CONSTANTTERMS(NCONSTANT(I))=.TRUE.
	    END DO
	  END IF
	  CALL LINFIT1D2(X,DATA,WEIGHT,NPOINTS,DEGREE,CONSTANTTERMS,
	1 COEFF,MAXNUMPARAM,ISHIFT,KPVT,DUMCOEFF,
	3 LEFTSIDE,UPTRIPART,AUX)
C  Note:  Slightly more accuracy can be obtained in the covariance matrix
C  if LEFTSIDE is recalculated, since roundoff error from scaling and 
C  unscaling will then not be present.
C
C  Determine the number of constant terms
	  NUMCONSTANT=0
	  DO I=1,DEGREE+1
	    IF (CONSTANTTERMS(I)) NUMCONSTANT=NUMCONSTANT+1
	  END DO
C  Compute the mean error for an observation of average weight
C  Determine the number of points with nonzero weight
	  NUM_DATA=0.0
	  DO I=1,NPOINTS
	    IF (WEIGHT(I).GT.0.0) NUM_DATA=NUM_DATA+1
	  END DO
C  Initialize the mean error
	  SIGMASUM=0.0
	  SUMWEIGHTS=0.0
	  DO I=1,NPOINTS
	    SIGMASUM=(POL(COEFF,DEGREE,X(I))-DATA(I))**2*WEIGHT(I)
	1   +SIGMASUM
	    SUMWEIGHTS=SUMWEIGHTS+WEIGHT(I)
	  END DO
	  AVWEIGHT=SUMWEIGHTS/NUM_DATA
	  IF (NUM_DATA-(DEGREE+1-NUMCONSTANT).GT.0) THEN
	    MEANERROR=SQRT(SIGMASUM*NUM_DATA/
	1   ((NUM_DATA-(DEGREE+1-NUMCONSTANT))*SUMWEIGHTS))
	  ELSE
	    MEANERROR=0.0
	  END IF
C  Calculate the covariance;  note that the matrix may be
C  smaller if some coefficients were held constant
	  CALL MATRIXINV(LEFTSIDE,MAXNUMPARAM,DEGREE+1-NUMCONSTANT,
	1 COVARIANCE,KPVT)
	  K=0
C  Scale the covariance matrix properly and make the matrix its
C  original size if necessary.  Start the loops from the high end
C  so as not to overwrite.  Note that the usage of ISHIFT is done
C  differently in this subroutine than in NONLINFIT.
	  DO NCOL=DEGREE+1,1,-1
	    IF (.NOT.CONSTANTTERMS(NCOL)) THEN
	      DO NROW=DEGREE+1,1,-1
	        IF (.NOT.CONSTANTTERMS(NROW)) THEN
	          COVARIANCE(NROW,NCOL)=MEANERROR**2*AVWEIGHT*
	1         COVARIANCE(NROW-ISHIFT(NROW),NCOL-ISHIFT(NCOL))
	        ELSE
	          COVARIANCE(NROW,NCOL)=0.0
	        END IF
	      END DO
	    ELSE
	      DO NROW=1,DEGREE+1
	        COVARIANCE(NROW,NCOL)=0.0
	      END DO
	    END IF
	  END DO
C  Evaluate the points in XPLOT
	  DO I=1,NPLPOINTS
	    YPLOT(I)=POL(COEFF,DEGREE,DBLE(XPLOT(I)))
	  END DO
	ELSE IF (TYPE.EQ.'S') THEN
	  DEGREE=2
	  IST=1
	  DO I=1,NPOINTS-2
	    DO K=1,3
	      IN=I+K-1
	      TEMPX(K)=X(IN)
	      TEMPDATA(K)=DATA(IN)
	      TEMPWEIGHT(K)=WEIGHT(IN)
	    END DO
C	    CALL LINFIT1D2(TEMPX,TEMPDATA,TEMPWEIGHT,3,DEGREE,
C	1   CONSTANTTERMS,COEFF)
	    CALL LINFIT1D2(TEMPX,TEMPDATA,TEMPWEIGHT,3,DEGREE,CONSTANTTERMS,
	1   COEFF,MAXNUMPARAM,KPVT,ISHIFT,DUMCOEFF,
	3   LEFTSIDE,UPTRIPART,AUX)
	    IEND=REALINDEX(XPLOT,NPLPOINTS,TEMPX(2))
	    IF (I.EQ.NPOINTS-2) IEND=NPLPOINTS
	    DO K=IST,IEND
	      YPLOT(K)=POL(COEFF,DEGREE,DBLE(XPLOT(K)))
	    END DO
	    IST=IEND+1
	  END DO
	  MEANERROR=0.0
	END IF
	RETURN
	END
