	SUBROUTINE LINFIT1D2(X,DATA,WEIGHT,NPOINTS,DEGREE,
	1CONSTANTTERMS,COEFF,MAXNUMPARAM,ISHIFT,KPVT,DUMCOEFF,
	2LEFTSIDE,UPTRIPART,AUX)
C
C       This subroutine helps LINFIT1D1.
C				Donald H. Gudehus, August, 1984
C			Version 2, Donald H. Gudehus, April 17, 2000
C			Reformed LEFTSIDE and UPTRIPART before returning
C
C	X	      (C) An array containing the values of the
C			  independent variable
C	DATA	      (C) An array containing the observations to
C			  be fitted
C	WEIGHT	      (C) An array for the weights of each point
C	NPOINTS	      (C) The dimensions of the above three arrays
C			  and the number of points to be fit
C	DEGREE	      (C) The degree of the polynomial fit
C	CONSTANTTERMS (C) An array which if .TRUE. holds that
C			  coefficient constant
C	COEFF	    (C/R) An array containing the coefficients of the
C			  polynomial of the fit.
C	MAXNUMPARAM   (C) The maximum number of parameters to be used
C			  in the calling program and the dimensions of
C			  COEFF and COVARIANCE in the calling program.
C
	INTEGER DEGREE,ORDER,ISHIFT(MAXNUMPARAM),KPVT(MAXNUMPARAM)
	LOGICAL CONSTANTTERMS(MAXNUMPARAM)
	REAL*8 X(NPOINTS),DATA(NPOINTS),WEIGHT(NPOINTS),
	1COEFF(MAXNUMPARAM),
	3DUMCOEFF(MAXNUMPARAM),LEFTSIDE(MAXNUMPARAM,MAXNUMPARAM),
	4UPTRIPART(MAXNUMPARAM*(MAXNUMPARAM+1)/2),AUX(MAX(MAXNUMPARAM-1,1)),
	5TERM
C
C  Initialize the arrays
	DO I=1,(DEGREE+1)*(DEGREE+2)/2
	  UPTRIPART(I)=0.0D0
	END DO
	DO I=1,DEGREE+1
	  DUMCOEFF(I)=0.0D0
	END DO
C  Determine the order of the equations after eliminating the constant
C  terms
	ORDER=0
	DO I=1,DEGREE+1
	  IF (.NOT.CONSTANTTERMS(I)) ORDER=ORDER+1
	END DO
C  Determine the shift between the location of a row or a column in
C  the full size matrix and the location in the smaller matrix
C  produced by eliminating rows and columns when some coefficients
C  are held constant.  Note that the calculation of and usage of ISHIFT
C  in this subroutine differs from that in NONLINFIT.
	ISHIFT(1)=0
	DO I=1,DEGREE
	  IF (CONSTANTTERMS(I)) THEN
	    ISHIFT(I+1)=ISHIFT(I)+1
	  ELSE
	    ISHIFT(I+1)=ISHIFT(I)
	  END IF
	END DO
C  Scale the data
	CALL SCALPOL(X,DATA,NPOINTS,KX,KY,COEFF,DEGREE,.TRUE.)
C  Form the sums for the left hand side of the normal equations and load
C  the array UPTRIPART which represents the upper triangular part of the
C  matrix for the left side of the normal equations
	L=1
	DO NCOL=1,DEGREE+1
	  IF (.NOT.CONSTANTTERMS(NCOL)) THEN
	    DO NROW=1,NCOL
	      IF (.NOT.CONSTANTTERMS(NROW)) THEN
	        NPOWER=NCOL+NROW-2
	        DO I=1,NPOINTS
	          TERM=1.0D0
	          DO NN=1,NPOWER
	            TERM=TERM*X(I)
	          END DO
	          UPTRIPART(L)=UPTRIPART(L)+TERM*WEIGHT(I)
	        END DO
	        LEFTSIDE(NROW-ISHIFT(NROW),NCOL-ISHIFT(NCOL))=
	1       UPTRIPART(L)
	        LEFTSIDE(NCOL-ISHIFT(NCOL),NROW-ISHIFT(NROW))=
	1       UPTRIPART(L)
	        L=L+1
	      END IF
	    END DO
	  END IF
	END DO
C  Compute the sums for the right hand side of the equations and load
C  them into the array DUMCOEFF which will later contain the solution.
	DO NROW=1,DEGREE+1
	  IF (.NOT.CONSTANTTERMS(NROW)) THEN
	    DO I=1,NPOINTS
	      TERM=1.0D0
	      DO NN=1,NROW-1
	        TERM=TERM*X(I)
	      END DO
	      DUMCOEFF(NROW-ISHIFT(NROW))=DUMCOEFF(NROW-ISHIFT(NROW))+
	1     TERM*DATA(I)*WEIGHT(I)
	    END DO
C  Loop through the columns to see if any column corresponds to a
C  coefficient which is to be held constant.  If so, subtract the
C  appropriate left side sum from the right side.
	    DO NCOL=1,DEGREE+1
	      IF (CONSTANTTERMS(NCOL)) THEN
	        NPOWER=NCOL+NROW-2
	        DO I=1,NPOINTS
	          TERM=1.0D0
	          DO NN=1,NPOWER
	            TERM=TERM*X(I)
	          END DO
	          DUMCOEFF(NROW-ISHIFT(NROW))=
	1         DUMCOEFF(NROW-ISHIFT(NROW))-TERM*WEIGHT(I)*
	2         COEFF(NCOL)
	        END DO
	      END IF
	    END DO
	  END IF
	END DO
C  Solve the equations
	EPS=1.E-16
	CALL SYMINV(DUMCOEFF,UPTRIPART,ORDER,1,EPS,IER,AUX)
C	IF (IER.GT.0) WRITE(6,1000) IER+1
C	IF (IER.GT.0) WRITE(7,1000) IER+1
1000	FORMAT(' Warning:  Possible loss of significance indicated at ',
	1'elimination step',I4/' in SYMINV routine, where pivot element ',
	1'was less than the internal tolerance'/ ' 1.E-16 set up in ',
	1'LINFIT1')
	IF (IER.EQ.-1) THEN
	  CALL MESSAGE('Attention SYMINV failed to solve '//
	1 'the system of linear equations')
	  RETURN
	END IF
C  Load COEFF with the solution, leaving any constant coefficients
C  unchanged
	DO I=1,DEGREE+1
	  IF (.NOT.CONSTANTTERMS(I)) THEN
	    COEFF(I)=DUMCOEFF(I-ISHIFT(I))
	  END IF
	END DO
C  Scale the data and solution back
	CALL SCALPOL(X,DATA,NPOINTS,KX,KY,COEFF,DEGREE,.FALSE.)
C  Reform LEFTSIDE and UPTRIPART.  LEFTSIDE can be used in the calling
C  program to solve for the covariance matrix.
	L=1
	DO NCOL=1,DEGREE+1
	  IF (.NOT.CONSTANTTERMS(NCOL)) THEN
	    DO NROW=1,NCOL
	      IF (.NOT.CONSTANTTERMS(NROW)) THEN
	        NPOWER=NCOL+NROW-2
	        DO I=1,NPOINTS
	          TERM=1.0D0
	          DO NN=1,NPOWER
	            TERM=TERM*X(I)
	          END DO
	          UPTRIPART(L)=UPTRIPART(L)+TERM*WEIGHT(I)
	        END DO
	        LEFTSIDE(NROW-ISHIFT(NROW),NCOL-ISHIFT(NCOL))=
	1       UPTRIPART(L)
	        LEFTSIDE(NCOL-ISHIFT(NCOL),NROW-ISHIFT(NROW))=
	1       UPTRIPART(L)
	        L=L+1
	      END IF
	    END DO
	  END IF
	END DO
C
	RETURN
	END
