	SUBROUTINE NLFUN10(X,MAXNPOINTS,DIMENSN_IND,PARAMETRS,CHOICE,
	1OPERATE,IX,FUNCTN,DIMENSN_DATA)
C
C       This subroutine helps NONLINFIT by selecting the chosen 
C  fitting function for a row of a FITS or SAD file or for an array of data, 
C  evaluating it taking into account a user defined operation if necessary, 
C  and returning the answer.  This subroutine is called by the SADFIT and
C  related subroutines (SADFIT_POWER_LAW and SADFIT_SPECTRUM) for the VMS 
C  version of MIIPS, and by FITSFIT and related subroutines (FITSFIT_POWER_LAW
C  and FITSFIT_SPECTRUM) for the Unix version of MIIPS, by FAKGAL1
C  and FAKGAL3, by PROBABLE2D, and by SUBSTAR.  If it is to be
C  used for other options, pay special attention to the use of COMMON blocks.
C  SCRATCH_BLOCK and SCRATCH_BLOCK2 are not available.
C			Donald H. Gudehus, January, 1987
C			Version 2, Donald H. Gudehus, February 18, 2008
C			Modified for FITS
C
C	X		(C)	The array containing the independent variable
C				values
C	MAXNPOINTS	(C)	The first dimension of the array X
C	DIMENSN_IND	(C)	The second dimension of the array X
C	PARAMETRS	(C)	The array containing the parameters used to
C				evaluate the function
C	CHOICE		(C)	The choice determining which function
C				to use within a category
C	OPERATE		(C)	A logical key which determines what to do.
C				If .FALSE., the function is evaluated, if 
C				.TRUE., a user defined operation is carried
C				out, e.g., convolution.
C	IX		(C)	An array containing the starting and ending
C				indexes for the evaluation of of FUNCTN
C	FUNCTN		(R)	The output array containing the function.
C	DIMENSN_DATA	(C)	The second dimension of the output array
C
	INTEGER DIMENSN_IND,DIMENSN_DATA,FUNCAT
	DIMENSION IX(2),X(MAXNPOINTS,DIMENSN_IND),PARAMETRS(1),
	1FUNCTN(MAXNPOINTS,DIMENSN_DATA)
	LOGICAL OPERATE,PLOT
	REAL*8 X,PARAMETRS,FUNCTN
	CHARACTER*(*) CHOICE
C
	REAL*8 TEMP
C
	INTEGER DEGREE2
	COMMON /FIT_SPECTRUM_BLOCK/ DEGREE2,NLINES
C
	CALL ALPHA(%REF(CHOICE),1,LEN(CHOICE),NCHAR,NFULL,NREST,1)
	PLOT=.FALSE.
	IF (CHOICE(NCHAR:NCHAR).EQ.'%') THEN
	  PLOT=.TRUE.
	  CHOICE(NCHAR:NCHAR)=' '
	END IF
	IF (OPERATE) THEN
	  CALL MESSAGE('NOT READY YET')
	ELSE
	  IF (CHOICE.EQ.'G') THEN !  Single Gaussian
	    DO N=IX(1),IX(2)
	      FUNCTN(N,1)=(X(N,1)-PARAMETRS(3))/PARAMETRS(2)
	      IF (ABS(FUNCTN(N,1)).GT.13.0) THEN
	        FUNCTN(N,1)=PARAMETRS(4) !  The constant term to avoid overflow
	      ELSE
	        FUNCTN(N,1)=PARAMETRS(1)/EXP(0.5*FUNCTN(N,1)**2)+PARAMETRS(4)
	      END IF
	    END DO
	  ELSE IF (CHOICE.EQ.'I') THEN !  Power law
	    DO N=IX(1),IX(2)
	      IF (.NOT.(PARAMETRS(2).LT.0.0.AND.X(N,1).EQ.0.0)) THEN
	        FUNCTN(N,1)=PARAMETRS(1)*X(N,1)**PARAMETRS(2)+PARAMETRS(3)
	      ELSE
	        FUNCTN(N,1)=0.0
	      END IF
	    END DO
	  ELSE IF (CHOICE.EQ.'GP') THEN !  Gaussian line profiles and polynomial
				        !  continuum
C  Calculate the continuum by proceeding from point to point
	    DO N=IX(1),IX(2)
	      FUNCTN(N,1)=POL(PARAMETRS,DEGREE2,X(N,1))
	    END DO
C  Calculate the modeled lines by proceeding from line to line
C  rather than point to point (since a single point receives contributions
C  from more than one line).  The absorption lines must be computed first,
C  since they multiply the continuum.  The emission lines are computed last
C  since they are simply added onto the existing intensity.
	    DO K=1,NLINES
C  Calculate the parameter numbers
	      N0=DEGREE2-1+K*3 !  The amplitude (emission line) or 
			       !  optical depth (absorption line)
	      N1=N0+1 !  The sigma
	      N2=N0+2 !  The offset in position
	      DO N=IX(1),IX(2)
	        TEMP=(X(N,1)-PARAMETRS(N2))/PARAMETRS(N1)
	        IF (ABS(TEMP).GT.13.0) THEN
		  !  Don't do anything
	        ELSE
	          IF (PARAMETRS(N0).GE.0.0) THEN !  Emission line
		    !  Don't do anything
	          ELSE !  Absorption line
	            FUNCTN(N,1)=FUNCTN(N,1)*EXP(PARAMETRS(N0)/
	1           EXP(0.5*TEMP**2))
	          END IF
	        END IF
	      END DO
	    END DO
	    DO K=1,NLINES
C  Calculate the parameter numbers
	      N0=DEGREE2-1+K*3 !  The amplitude (emission line) or 
			       !  optical depth (absorption line)
	      N1=N0+1 !  The sigma
	      N2=N0+2 !  The offset in position
	      DO N=IX(1),IX(2)
	        TEMP=(X(N,1)-PARAMETRS(N2))/PARAMETRS(N1)
	        IF (ABS(TEMP).GT.13.0) THEN
		  !  Don't do anything
	        ELSE
	          IF (PARAMETRS(N0).GE.0.0) THEN !  Emission line
	            FUNCTN(N,1)=FUNCTN(N,1)+PARAMETRS(N0)/
	1           EXP(0.5*TEMP**2)
	          ELSE !  Absorption line
		    !  Don't do anything
	          END IF
	        END IF
	      END DO
	    END DO
	  END IF
	END IF
C
	RETURN
	END
