	SUBROUTINE POLINTERP1D(ARRAY1,ARRAY2,NPOINTS,VALUE1,VALUE2,DEGREE,
	1MODE,COEFF)
C
C       This subroutine performs a polynomial interpolation.  ARRAY1 and
C  ARRAY2 contain the independent and dependent values, respectively used
C  in the polynomial fit.  In its usual usage, ARRAY1 and ARRAY2 contain 
C  many more points than are used in fitting a typical polynomial.  The 
C  subroutine automatically selects those points which bracket the given 
C  input value and its corresponding interpolated value.  Equal weighting 
C  is assumed.  The interval between values in ARRAY1 is allowed to be 
C  arbitrary, however, it is assumed that the values in ARRAY1 are 
C  monotonically increasing or decreasing.  If the intervals in the array 
C  are unequal, less than optimal results may occur.  In that case it may 
C  be necessary to try and equalize the intervals by taking the log, for 
C  example.
C       For future development, other interpolation schemes will be included.
C
C					Donald H. Gudehus, October, 1986
C
C	ARRAY1	(C)	The array containing the independent values
C	ARRAY2	(C)	The array containing the dependent values
C	NPOINTS	(C)	The dimension of ARRAY1 and ARRAY2 and the
C			number of valid data points.
C	VALUE1	(C)	The known value within the range of ARRAY1
C	VALUE2	(R)	The requested value within the range of ARRAY2
C	DEGREE	(C)	The degree of the polynomial to be used
C	MODE	(C)	The mode of operation:
C			Units position:
C			  0  No constraints
C			  1  The slope at the beginning of ARRAY2 is 
C			     desired to be be zero
C			Tens position:
C			  0  No transformation for X
C			  1  Take the log of X first
C			Hundreds position:
C			  0  No transformation for Y
C			  1  Take the log of Y first and convert back
C	COEFF	(R)	The coefficients of the polynomial fit
C
	PARAMETER (MAXNUMPARAM=11)  ! 10th degree polynomial maximum
	DIMENSION ARRAY1(NPOINTS),ARRAY2(NPOINTS)
	INTEGER ADDRESS1,ADDRESS2,DEGREE
	REAL*8 COEFF(MAXNUMPARAM)
C
C  Get some dynamic memory.  Eventually this will be extended to accomadate
C  any degree.
	CALL DYNAMIC(ADDRESS1,NPOINTS+1,'  R8',.TRUE.)
	CALL DYNAMIC(ADDRESS2,NPOINTS+1,'  R8',.TRUE.)
	CALL POLINTERP1D1(%VAL(ADDRESS1),%VAL(ADDRESS2),ARRAY1,ARRAY2,
	1NPOINTS,VALUE1,VALUE2,DEGREE,MODE,COEFF)
	CALL DYNAMIC(ADDRESS1,NPOINTS+1,'  R8',.FALSE.)
	CALL DYNAMIC(ADDRESS2,NPOINTS+1,'  R8',.FALSE.)
C
	RETURN
	END
