	SUBROUTINE POLINTERP1D1(ARRAY1WORK,ARRAY2WORK,ARRAY1,ARRAY2,
	1NPOINTS,VALUE1,VALUE2,DEGREE,MODE,COEFF)
C
C       This subroutine performs a polynomial interpolation.  ARRAY1 and
C  ARRAY2 contain the independent and dependent values, respectively used
C  in the polynomial fit.  In its usual usage, ARRAY1 and ARRAY2 contain
C  many more points than are used in fitting a typical polynomial.  The
C  subroutine automatically selects those points which bracket the given
C  input value and its corresponding interpolated value.  Equal weighting
C  is assumed.  The interval between values in ARRAY1 is allowed to be
C  arbitrary, however, it is assumed that the values in ARRAY1 are
C  monotonically increasing or decreasing.  If the intervals in the array
C  are unequal, less than optimal results may occur.  In that case it may
C  be necessary to try and equalize the intervals by taking the log, for
C  example.
C
C			Donald H. Gudehus, October, 1986
C			Version 2, Donald H. Gudehus, December 1, 1994
C			Tidied up logic
C			Version 3, Donald H. Gudehus, May, 1995
C			Expanded definition of MODE
C
C	ARRAY1	(C)	The independent variable array
C	ARRAY2	(C)	The dependent variable array
C	NPOINTS	(C)	The dimension of ARRAY1 and ARRAY2 and the
C			number of valid data points.
C	VALUE1	(C)	The known value within the range of ARRAY1
C	VALUE2	(R)	The requested value within the range of ARRAY2
C	DEGREE	(C)	The degree of the polynomial fit
C	MODE	(C)	The mode of operation:
C			Units position:
C			  0  No constraints
C			  1  The slope at the beginning of ARRAY2 is 
C			     desired to be be zero
C			Tens position:
C			  0  No transformation for X
C			  1  Take the log of X first
C			Hundreds position:
C			  0  No transformation for Y
C			  1  Take the log of Y first and convert back
C	COEFF	(R)	The coefficients of the polynomial fit
C
	DIMENSION ARRAY1WORK(NPOINTS+1),ARRAY2WORK(NPOINTS+1),
	1ARRAY1(NPOINTS),ARRAY2(NPOINTS)
	PARAMETER (MAXNUMPARAM=11)  ! 10th degree polynomial maximum
	INTEGER DEGREE,DUMDEGREE,C1000
	LOGICAL NOTYET,EXACT_MATCH,EVEN,CONSTANTTERMS(MAXNUMPARAM),
	1SLOPE_ZERO,LOG_X,LOG_Y
	REAL*4 ARRAY1,ARRAY2,VALUE1,VALUE2,VALUE2TEMP
	REAL*8 ARRAY1WORK,ARRAY2WORK,CENTERPOINT,
	1WEIGHT(MAXNUMPARAM),COEFF(MAXNUMPARAM),MEANERROR,
	2COVARIANCE(MAXNUMPARAM,MAXNUMPARAM)
C
C  Get the modes of operation
	CALL DECODE_DECIMAL(MODE,C1000,LOG_Y,LOG_X,SLOPE_ZERO)
	DUMDEGREE=DEGREE
	IF (FLOAT(DUMDEGREE)/2.0-INT(DUMDEGREE/2).GT.0.0) THEN
	  EVEN=.FALSE.
	ELSE
	  EVEN=.TRUE.
	END IF
C  Initialize the arrays for LINFIT1D
	DO I=1,MAXNUMPARAM
	  CONSTANTTERMS(I)=.FALSE.
	  WEIGHT(I)=1.0D0
	END DO
C  Load ARRAY1WORK and ARRAY2WORK
	IF (LOG_X) THEN
	  DO I=1,NPOINTS
	    ARRAY1WORK(I)=LOG10(ARRAY1(I))
	  END DO
	  DUMVALUE1=LOG10(VALUE1)
	ELSE
	  DO I=1,NPOINTS
	    ARRAY1WORK(I)=ARRAY1(I)
	  END DO
	  DUMVALUE1=VALUE1
	END IF
	IF (LOG_Y) THEN
	  DO I=1,NPOINTS
	    ARRAY2WORK(I)=LOG10(ARRAY2(I))
	  END DO
	ELSE
	  DO I=1,NPOINTS
	    ARRAY2WORK(I)=ARRAY2(I)
	  END DO
	END IF
C  Protect if reading beyond region of valid data
	IF (NPOINTS.GT.1) THEN
	  ARRAY1WORK(NPOINTS+1)=2.0*ARRAY1(NPOINTS)-ARRAY1(NPOINTS-1)
	  ARRAY2WORK(NPOINTS+1)=2.0*ARRAY2(NPOINTS)-ARRAY2(NPOINTS-1)
	ELSE
	  ARRAY1WORK(NPOINTS+1)=ARRAY1(NPOINTS)
	  ARRAY2WORK(NPOINTS+1)=ARRAY2(NPOINTS)
	END IF
	NOTYET=.TRUE.
	I=1
	EXACT_MATCH=.FALSE.
	DO WHILE (NOTYET.AND.I.LE.NPOINTS)  ! Checks all points for a possible
					    ! match
	  IF (ARRAY1WORK(I).EQ.DUMVALUE1) THEN
	    VALUE2=ARRAY2WORK(I)
	    INDEX=I
	    NOTYET=.FALSE.
	    EXACT_MATCH=.TRUE.
C  We have a match but we need to get the coefficients
	  END IF
	  I=I+1
	END DO
C
	IF (.NOT.EXACT_MATCH) THEN
	  NOTYET=.TRUE.
	  I=2
	  DO WHILE (NOTYET.AND.I.LE.NPOINTS)
	    IF (ARRAY1WORK(2).GT.ARRAY1WORK(1)) THEN  ! Monotonically 
						      ! increasing
	      IF (ARRAY1WORK(I).GE.DUMVALUE1) THEN
	        IF (EVEN) THEN  ! Degree is even
	          CENTERPOINT=(ARRAY1WORK(I)+ARRAY1WORK(I-1))/2
	          IF (DUMVALUE1.GT.CENTERPOINT) THEN
	            INDEX=I-DUMDEGREE/2
	          ELSE
	            INDEX=I-DUMDEGREE/2-1
	          END IF
	        ELSE  ! Not even
	          INDEX=I-(DUMDEGREE+1)/2
	        END IF
	        NOTYET=.FALSE.
	      END IF
	    ELSE  ! Monotonically decreasing
	      IF (ARRAY1WORK(I).LE.DUMVALUE1) THEN
	        IF (EVEN) THEN
	          CENTERPOINT=(ARRAY1WORK(I)+ARRAY1WORK(I-1))/2
	          IF (DUMVALUE1.LT.CENTERPOINT) THEN
	            INDEX=I-DUMDEGREE/2
	          ELSE
	            INDEX=I-DUMDEGREE/2-1
	          END IF
	        ELSE  ! Not even
	          INDEX=I-(DUMDEGREE+1)/2
	        END IF
	        NOTYET=.FALSE.
	      END IF
	    END IF
	    I=I+1
	  END DO
	  IF (NOTYET) INDEX=NPOINTS-DUMDEGREE
	END IF
	INDEX=MIN(NPOINTS-1,MAX(1,INDEX))
	NPOINTSLF=DUMDEGREE+1
	DO WHILE (INDEX+NPOINTSLF-1.GT.NPOINTS+1)
	  DUMDEGREE=DUMDEGREE-1
	  NPOINTSLF=NPOINTSLF-1
	END DO
	IF (SLOPE_ZERO) THEN
C  Make sure slope is zero at INDEX=1
	  IF (INDEX.EQ.1) THEN
	    CONSTANTTERMS(2)=.TRUE.
	    COEFF(2)=0.0
	  END IF
	END IF
C  Use LINFIT1D to get a polynomial fit
	CALL LINFIT1D(ARRAY1WORK(INDEX),ARRAY2WORK(INDEX),
	1WEIGHT,NPOINTSLF,DUMDEGREE,CONSTANTTERMS,COEFF,COVARIANCE,
	2MAXNUMPARAM,MEANERROR,DUMVALUE1,VALUE2TEMP,1)
	CONSTANTTERMS(2)=.FALSE.
	IF (.NOT.EXACT_MATCH) VALUE2=VALUE2TEMP
	IF (LOG_Y) VALUE2=10**VALUE2
C
	RETURN
	END
