	SUBROUTINE PROBABLE2D(DATA,NX,NY,IBIN,NBINS,EXCLUDE,PROBABLE_VAL,
	1SIGMA)
C
C       This subroutine returns the most probable value and mean error of a
C  two-dimensional array of data by fitting a gaussian to the histogram of
C  array values.  By fitting mostly on the low intensity side of the peak,
C  the influence of background objects, e.g., stars and cosmic rays, on 
C  the most probable value and mean error is minimized.  This subroutine is 
C  called by PROBABLE2DS which is called by STATS, MFLAT and SKYBACK.
C				Donald H. Gudehus, September, 1982
C				Version 2, Donald H. Gudehus, October, 1987
C				Incorporated a call to nonlinfit to fit a
C				gaussian.
C				Version 3, Donald H. Gudehus, December, 1988
C				Added check for excessive number of zeroes in
C				histogram, such as might occur if the data is 
C				integerized and the sigma is small.
C				Version 3U, Donald H. Gudehus, January 25, 2008
C				Converted code to work under Unix.
C
C	DATA		(C)	The array
C	NX		(C)	Dimension in the X direction and the number of
C				pixels to be read
C	NY		(C)	Dimension in the Y direction and the number of
C				pixels to be read
C	EXCLUDE		(C)	A key such that when true, values of zero are
C				excluded from the computation
C	PROBABLE_VAL	(R)	The most probable value of the pixels in the
C				array
C	SIGMA		(R)	The mean error
C
	DIMENSION DATA(NX,NY),IBIN(NBINS)
	REAL*8 PROBABLE_VAL,SIGMA
	LOGICAL EXCLUDE
C
	INTEGER DIMENSN_DATA
	PARAMETER (DIMENSN_DATA=1,MAXSIZE=200,MAXNUMPARAMGAUSS=5)
	INTEGER*4 LOBIN,HIBIN,IX(2)
	LOGICAL CONSTANTTERMSGAUSS(MAXNUMPARAMGAUSS),INTERMEDIATE,OPERATE
	REAL XPLOTGAUSS(MAXSIZE,1),DATPLOTGAUSS(MAXSIZE,1)
	REAL*8 EFFECTIVE_PROBABLE_VAL,EFFECTIVE_SIGMA,SCALE,AVERAGE,
	1OFFSET,XGAUSS(MAXSIZE,1),
	2DATAGAUSS(MAXSIZE,1),WEIGHTGAUSS(MAXSIZE,1),
	3PARAMETRSGAUSS(MAXNUMPARAMGAUSS),
	3COVARIANCEGAUSS(MAXNUMPARAMGAUSS,MAXNUMPARAMGAUSS),
	4MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	5REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXSIZE,DIMENSN_DATA)
	CHARACTER*1 ITERATION_SCHEME
	CHARACTER*6 CHOICE
C
C  Get the simple average and sigma
	CALL AVERSIG(DATA,NX,NY,1,1,NX,NY,EXCLUDE,AVERAGE,SIGMA)
	IF (SIGMA.LE.0.0) CALL ERROR(-3,'Sigma is zero')
C  Scale the array so that the effective sigma of amplitude is 50, but
C  decrease this if the number of points is small.
	NUMDATAPOINTS=NX*NY
	EFFECTIVE_SIGMA=50.0D0*MIN(1.0,SQRT((1.0*NUMDATAPOINTS)/1000.0))
	SCALE=EFFECTIVE_SIGMA/SIGMA
C  Offset the array so that the effective probable value is 200
	EFFECTIVE_PROBABLE_VAL=200.0D0
	OFFSET=EFFECTIVE_PROBABLE_VAL-AVERAGE*SCALE
	DO J=1,NY
	  DO I=1,NX
	    DATA(I,J)=DATA(I,J)*SCALE+OFFSET
	  END DO
	END DO
C  Inititalize
	PROBABLE_VAL=AVERAGE
  10	EFFECTIVE_PROBABLE_VAL=PROBABLE_VAL*SCALE+OFFSET
C  Take the range of pixel values to bin as minus 3.0*EFFECTIVE_SIGMA and 
C  plus 2.0*EFFECTIVE_SIGMA about the average.  LOBIN and HIBIN are index
C  offsets in units of integerized scaled physical value.
	LOBIN=MAX(1,INT(EFFECTIVE_PROBABLE_VAL-3.0*EFFECTIVE_SIGMA))
	HIBIN=MIN(NBINS,INT(EFFECTIVE_PROBABLE_VAL+2.0*EFFECTIVE_SIGMA))
	NUMBINS=HIBIN-LOBIN+1
C  Initialize the array IBIN (the bins of the histogram in 1 AD unit)
	DO I=1,NUMBINS
	  IBIN(I)=0
	END DO
C  Nested DO loops to cover the area under consideration
	DO J=1,NY
	  DO 40 I=1,NX
C  The index of the bins will be the same as the AD unit minus
C  a constant.  Take the AD unit of the pixel and turn it
C  into an integer.
	    NBIN=NINT(DATA(I,J))
C  Test to see if the value (in AD units) is in the desired 
C  range.  If so, add 1 to the bin for that value (the integer value of 
C  AD units minus the low value plus 1).
	    IF(NBIN.GE.LOBIN.AND.NBIN.LE.HIBIN)
	1   IBIN(NBIN-LOBIN+1)=IBIN(NBIN-LOBIN+1)+1
  40	  CONTINUE
  	END DO
C  Find the maximum of the histogram
	MAXNUM=0
	INDEX=1
C  Count the number of bins with zero
	NUMZERO=0
	DO I=1,NUMBINS
	  IF (IBIN(I).GT.MAXNUM) THEN
	    MAXNUM=IBIN(I)
	    INDEX=I
	  END IF
	  IF (IBIN(I).EQ.0) NUMZERO=NUMZERO+1
	END DO
C  Get set up to fit
	IX(1)=1
	IX(2)=INT(INDEX+0.6*EFFECTIVE_SIGMA)
	DO I=IX(1),IX(2)
	  XGAUSS(I,1)=I+0.5  ! Offsets to center of bin
	  DATAGAUSS(I,1)=IBIN(I)
	END DO
	DO I=IX(1),IX(2)
	  WEIGHTGAUSS(I,1)=1.0D0
	END DO
	CHOICE='G'
	CONSTANTTERMSGAUSS(1)=.FALSE.
	CONSTANTTERMSGAUSS(2)=.FALSE.
	CONSTANTTERMSGAUSS(3)=.FALSE.
	CONSTANTTERMSGAUSS(4)=.TRUE.
	PARAMETRSGAUSS(1)=MAXNUM  ! Peak value
	PARAMETRSGAUSS(2)=EFFECTIVE_SIGMA
	PARAMETRSGAUSS(3)=INDEX+0.5  ! X location
	PARAMETRSGAUSS(4)=0.0  ! Constant term
	NUMPARAM=4
	ITERATION_SCHEME='C'
	TOLERANCE=0.0005
	INTERMEDIATE=.FALSE.
	OPERATE=.FALSE.
	CALL NONLINFIT(XGAUSS,MAXSIZE,1,DATAGAUSS,1,WEIGHTGAUSS,
	2IX,10,CHOICE,CONSTANTTERMSGAUSS,PARAMETRSGAUSS,MAXNUMPARAMGAUSS,
	3NUMPARAM,ITERATION_SCHEME,TOLERANCE,INTERMEDIATE,OPERATE,
	4COVARIANCEGAUSS,MEANERROR,MEANERRORMEANERROR,REDCHISQ,AVREDCHISQ,
	5XPLOTGAUSS,DATPLOTGAUSS,1,RESIDUALS)
	IF (PARAMETRSGAUSS(1).LE.0.0.OR.PARAMETRSGAUSS(2).LE.0.0) THEN
	  CALL MESSAGE('Bad statistics in this region, try again')
	  EFFECTIVE_PROBABLE_VAL=OFFSET
	  EFFECTIVE_SIGMA=0.0
	  RETURN
	ELSE
	  EFFECTIVE_PROBABLE_VAL=PARAMETRSGAUSS(3)+LOBIN-1.0
	  EFFECTIVE_SIGMA=PARAMETRSGAUSS(2)
	END IF
C  Unscale the results
	PROBABLE_VAL=(EFFECTIVE_PROBABLE_VAL-OFFSET)/SCALE
	SIGMA=EFFECTIVE_SIGMA/SCALE
	RETURN
	END
