	SUBROUTINE FINDCOORD(XARRAY,YARRAY,IFOUND,NSIZE,NPOINTS,KEY)
C
C       This subroutine returns the indexes of the physical coordinates
C  stored in the arrays XARRAY and YARRAY which are closest to the 
C  locations selected by the cursor or crosshair on a plotting device such 
C  as a graphics terminal and which were placed in the COMMON block PLOT_BLOCK.
C				Donald H. Gudehus, 1986
C				Version 2, Donald H. Gudehus, January, 1990
C				Added argument for KEY
C				Version 3, Donald H. Gudehus, January 9, 2000
C				Changed END from 7FFFFFFF to FF7FFFFF so as
C				to be compatible with the Unix version.
C
C	XARRAY	(C)	The array of x values which were plotted and from
C			which a cursor marked position or positions are to
C			be derived.
C	YARRAY	(C)	The array of y values which were plotted and from
C			which a cursor marked position or positions are to
C			be derived.
C	IFOUND	(R)	An array containing the indexes which were found 
C			for the cursor marked points.
C	NSIZE	(C)	The size of the above arrays.
C	NPOINTS	(R)	The number of cursor marked points
C	KEY	(C)	A key which controls what is to be done.  If KEY
C			equals 0, only the X array is searched.  If KEY 
C			equals 1, only the Y array is searched.  If KEY
C			equals 2, both arrays are searched.
C
	DIMENSION XARRAY(NSIZE),YARRAY(NSIZE),IFOUND(NSIZE)
C
	COMMON /PLOT_BLOCK/ XCOORD(1000),YCOORD(1000)
	DATA END/'FF7FFFFF'X/
C
	NPOINTS=NEL(XCOORD,1000)
	IF (KEY.EQ.0) THEN  ! Searches XARRAY only
C  The arrays XARRAY and YARRAY may contain values equal to END so that
C  those points will not be plotted.  Do not compute a distance for those
C  points.
	  DO N=1,NPOINTS  ! Loops through the cursor selected points
C  Get a starting value for the minimum distance for each selected point
	    I=1
	    DO WHILE(XARRAY(I).EQ.END.AND.I.LE.NSIZE)
	      I=I+1
	    END DO
	    CLOSEST_DISTANCE=ABS(XARRAY(I)-XCOORD(1))
C  Inititalize IFOUND
	    IFOUND(N)=I
	    DO NN=I,NSIZE  ! Loops through the arrays for the plotted points
	      IF (XARRAY(NN).NE.END) THEN
	        TEST=ABS(XARRAY(NN)-XCOORD(N))
	        IF (TEST.LT.CLOSEST_DISTANCE) THEN
	          CLOSEST_DISTANCE=TEST
	          IFOUND(N)=NN
	        END IF
	      END IF
	    END DO
	  END DO
	ELSE IF (KEY.EQ.1) THEN  ! Searches YARRAY only
	  DO N=1,NPOINTS  ! Loops through the cursor selected points
C  Get a starting value for the minimum distance for each selected point
	    I=1
	    DO WHILE(YARRAY(I).EQ.END.AND.I.LE.NSIZE)
	      I=I+1
	    END DO
	    CLOSEST_DISTANCE=ABS(YARRAY(I)-YCOORD(1))
C  Inititalize IFOUND
	    IFOUND(N)=I
	    DO NN=I,NSIZE  ! Loops through the arrays for the plotted points
	      IF (YARRAY(NN).NE.END) THEN
	        TEST=ABS(YARRAY(NN)-YCOORD(N))
	        IF (TEST.LT.CLOSEST_DISTANCE) THEN
	          CLOSEST_DISTANCE=TEST
	          IFOUND(N)=NN
	        END IF
	      END IF
	    END DO
	  END DO
	ELSE IF (KEY.EQ.2) THEN  ! Searches both arrays
	  DO N=1,NPOINTS  ! Loops through the cursor selected points
C  Get a starting value for the minimum distance for each selected point
	    I=1
	    DO WHILE((XARRAY(I).EQ.END.OR.YARRAY(I).EQ.END).AND.I.LE.NSIZE)
	      I=I+1
	    END DO
	    CLOSEST_DISTANCE=SQRT((XARRAY(I)-XCOORD(1))**2+
	1   (YARRAY(I)-YCOORD(1))**2)
C  Inititalize IFOUND
	    IFOUND(N)=I
	    DO NN=I,NSIZE  ! Loops through the arrays for the plotted points
	      IF (XARRAY(NN).NE.END.AND.YARRAY(NN).NE.END) THEN
	        TEST=SQRT((XARRAY(NN)-XCOORD(N))**2+
	1       (YARRAY(NN)-YCOORD(N))**2)
	        IF (TEST.LT.CLOSEST_DISTANCE) THEN
	          CLOSEST_DISTANCE=TEST
	          IFOUND(N)=NN
	        END IF
	      END IF
	    END DO
	  END DO
	END IF
	RETURN
	END
	    
