	SUBROUTINE RADGAL
C
C       This subroutine reads a data file containing a galaxy's surface
C  brightness as a function of radius and then allows one to plot a
C  preliminary Hubble "a" against the radius and solve for the coefficients
C  describing a chosen surface brightness law.  The solution is made with the
C  nonlinear least squares subroutine, nonlinfit.  The data are weighted and
C  the mean errors of each parameter are calculated.  One may convolve the
C  surface brightness law by a seeing function obtained from a data file
C  before doing the fit.  The total light of the galaxy is calculated next
C  and the residuals of the solution plotted.
C			Donald H. Gudehus, December, 1982
C			Version 2, Donald H. Gudehus, October, 1986
C			Incorporated NONLINFIT
C			Version 2U, Donald H. Gudehus, February, 2008
C			Modified code for Unix
C
	INCLUDE 'miipspar.for'
C
	INTEGER DIMENSN_IND,DIMENSN_DATA
	PARAMETER (MAXNUMPARAM=8,DIMENSN_IND=1,DIMENSN_DATA=1)
C
	CHARACTER*1 METHOD,WEIGHT_SCHEME,ITERATION_SCHEME
	CHARACTER*6 CHOICE,IDLAST,IDREAD
	CHARACTER*50 XLABEL,YLABEL,TITLE
	CHARACTER*80 FILENAMESMEAR,FILENAMEWEIGHTS,FILENAMERESULTS
C
	INTEGER COLOR
	INTEGER FUNCAT,IRADII(2),NPLOTPOINTS,LINVECT,INC,
	1LINTYPE,NSYMBOL
C
	LOGICAL CONSTANTTERMS(MAXNUMPARAM),INTERMEDIATE,CONVOLVE,
	1INVERT(2),INTERACT,CROSSHAIRS,
	2ITERATE,FIRSTIME,PLOTD,PLOTA,ADJUST,MORE,
	3EVALUATE,POWER,PARAMETRSCHANGED,GO,STORERESULTS,STOREOPENED
C
	REAL ECCENTRICITY(MAXDIAG),SMEARREAD(MAXDIAG),
	3SMEARRADIUS(MAXDIAG),
	5FILEWEIGHT(MAXDIAG),READOUT(2),
	4XPLOT(MAXDIAG),DATPLOT(MAXDIAG),
	5OBMINUSCALC(MAXDIAG,2),INCLINA,LOGINTADU,NUMRADII,
	6SIGMAPARAM(MAXNUMPARAM)
C
	REAL*8 RADIUS(MAXDIAG,DIMENSN_IND),
	1SURFBRIGHTOB(MAXDIAG,DIMENSN_DATA),
	1WEIGHT(MAXDIAG,DIMENSN_DATA),
	1PARAMETRS(MAXNUMPARAM),COVARIANCE(MAXNUMPARAM,MAXNUMPARAM),
	2MEANERROR(DIMENSN_DATA),MEANERRORMEANERROR(DIMENSN_DATA),
	3REDCHISQ(DIMENSN_DATA),AVREDCHISQ,RESIDUALS(MAXDIAG,DIMENSN_DATA),
	2SKY,ADUINTEGRAL1,ADUINTEGRAL2,SIGMASUM_MAG,SUMWEIGHT
C
C
	INTEGER NPOINTS_S,NPOINTS_FIT,NPOINTS_C,IRADIICONVOLVE(2)
	COMMON /NLF1_INTEGER_SCRATCH_BLOCK/ NPOINTS_S,NPOINTS_FIT,NPOINTS_C,
	1IRADIICONVOLVE
C
	REAL*4 RADIUSINCONVOLVE(MAXDIAG),SURFBRIGHTNOTCONVOLVE(MAXDIAG),
	1SMEAR(MAXDIAG),SURFBCONVOLVE(MAXDIAG),RADII_MINMAX(2),CONVOLVEFACTOR,
	2RADIUS_C
	COMMON /NLF1_REAL4_SCRATCH_BLOCK/ RADIUSINCONVOLVE,
	1SURFBRIGHTNOTCONVOLVE,SMEAR,SURFBCONVOLVE,RADII_MINMAX,
	2CONVOLVEFACTOR,RADIUS_C
C
	CHARACTER*6 ID,ID2
	COMMON /ID/ ID,ID2
C
	LOGICAL OFIL
	COMMON /OLDFILE/ OFIL
C
	CHARACTER*80 DCNAME
	COMMON /CFILE/ DCNAME
C
	DATA PI /3.14159265/,TWOPI /6.2831853/
C
	CALL MESSAGE('RADGAL, Version 2')
	FUNCAT=1
	ITERATION_SCHEME=' '
	TOLERANCE=0.0
	INTERMEDIATE=.TRUE.
	STOREOPENED=.FALSE.
	ADJUST=.FALSE.
C  Get the data to be fitted
	IDLAST=' '
	CALL MESSAGE('Open the data file')
	CALL OPENCF(21,1,'FILE_21=',0)
   2	REWIND 21 !  Return point for additional fitting
	CALL USRINP(%REF(ID),1,'A',6,1,'ID=',
	1'Enter the ID of the object:  (default = last galaxy entry)')
	CALL ALPHA(%REF(ID),1,6,LX,NF,NR,2)
	IF (ID.EQ.IDLAST) GO TO 42
	READ(21,1100,ERR=14)
	GO TO 16
  14	CALL MESSAGE('Couldn''t read the header')
	RETURN
  16	CONTINUE
1100	FORMAT(' ','ID',8X,'COLUMN',5X,'ROW',1X,'INCLINATION',
	11X,'ECCENTRICITY',1X,'RADIUS',5X,'ADU',3X,'LOG(INTEGRAL)')
	IDREAD=' '
	DO WHILE (ID.NE.IDREAD)
	  READ(21,1101,END=22) IDREAD,COLUMN,ROW,INCLINA,ECCENTRICITY(1),
	1 RADIUS(1,1),SURFBRIGHTOB(1,1),LOGINTADU
1101	  FORMAT(' ',A6,1X,F9.3,1X,F9.3,1X,F7.2,6X,F6.3,F8.3,1X,
	1 F13.6,1X,F11.8)
	END DO
	IF (RADIUS(1,1).NE.0.0) CALL MESSAGE('Warning!  The file does not '//
	1'begin at the center of the galaxy')
	GO TO 24
  22	CALL MESSAGE('Couldn''t find the object')
	RETURN
  24	CONTINUE
	NPOINTS_FILE=1
	DO WHILE (ID.EQ.IDREAD)
	  NPOINTS_FILE=NPOINTS_FILE+1
	  READ(21,1101,END=42) IDREAD,COLUMN,ROW,INCLINA,
	1 ECCENTRICITY(NPOINTS_FILE),RADIUS(NPOINTS_FILE,1),
	1 SURFBRIGHTOB(NPOINTS_FILE,1),LOGINTADU
	END DO
  42	NPOINTS_FILE=NPOINTS_FILE-1 !  Ignores the last number read
C  Load the plotting array
	DO I=1,NPOINTS_FILE
	  XPLOT(I)=RADIUS(I,1)
	END DO
C  See if a first estimate of the Hubble "a" is desired
	CALL MESSAGE('Do you wish to plot the first estimate of the '//
	1'Hubble "a" against radius?')
	PLOTA=.FALSE.
	CALL USRINP(PLOTA,1,'L',1,1,'PLOT=','(default = N)')
C  Set up the plotting parameters
	NPLOTS=1
	LINVECT=0
	INC=1
	LINTYPE=0
	NSYMBOL=0
	INVERT(1)=.FALSE.
	INVERT(2)=.FALSE.
	COLOR=16776960
	INTERACT=.FALSE.
	CROSSHAIRS=.TRUE.
	XLABEL='RADIUS (pixels)'
	YLABEL='HUBBLE A (pixels)'
	TITLE='FIRST ESTIMATE'
	SKY=SURFBRIGHTOB(1,1)
	DO I=1,NPOINTS_FILE
	  SKY=MIN(SKY,SURFBRIGHTOB(I,1))
	END DO
	IF (PLOTA) THEN
	  CALL USRINP(SKY,1,'F',8,3,'SKY=',
	1 'Enter the sky level to be subtracted from the data: '//
	2 '(default = minimum)')
	  DO 50 I=2,NPOINTS_FILE !  We must start at 2 to avoid division by 0
	    DENOM=SURFBRIGHTOB(I,1)-SKY
	    SIGN=1.0
	    IF (DENOM.EQ.0.0) THEN
	      DATPLOT(I)=0.0
	      GO TO 50
	    ELSE IF (DENOM.LT.0.0) THEN
	      DENOM=-DENOM
	      SIGN=-1.0
	    END IF
	    FACTOR=(SURFBRIGHTOB(1,1)-SKY)/DENOM
	    DATPLOT(I)=SIGN*RADIUS(I,1)/(SQRT(FACTOR)-1.0)
  50	  CONTINUE
	  NPLOTPOINTS=NPOINTS_FILE-1
	  CALL XYPLOT(XPLOT(2),DATPLOT(2),MAXDIAG-1,NPLOTS,
	1 NPLOTPOINTS,LINVECT,INC,LINTYPE,NSYMBOL,INVERT,COLOR,INTERACT,
	2 CROSSHAIRS,XLABEL,YLABEL,TITLE,' ',.FALSE.)
	  CALL RESETTEK
	END IF
	CHOICE=' '
	DO WHILE (CHOICE.NE.'G'.AND.CHOICE.NE.'H'.AND.CHOICE.NE.'M'
	1.AND.CHOICE.NE.'MC'.AND.CHOICE.NE.'ML'.AND.CHOICE.NE.'MR'
	2.AND.CHOICE.NE.'P'.AND.CHOICE.NE.'T'.AND.CHOICE.NE.'V'
	3.AND.CHOICE.NE.'VC'.AND.CHOICE
	4.NE.'K'.AND.CHOICE.NE.'L'.AND.CHOICE.NE.'HV'.AND.CHOICE.NE.'I'
	5.AND.CHOICE.NE.'SI'.AND.CHOICE.NE.'E3')
	  CHOICE='M'
	  CALL MESSAGE('Enter the surface brightness law:')
	  CALL MESSAGE('     G  = Gaussian')
	  CALL MESSAGE('     H  = Hubble')
	  CALL MESSAGE('     HV = Hubble with variable power in
	1 denominator')
	  CALL MESSAGE('     I  = Inverse radius')
	  CALL MESSAGE('     K  = Constant')
	  CALL MESSAGE('     L  = Straight line')
	  CALL MESSAGE('     M  = Modified Hubble (default)')
	  CALL MESSAGE('     MC = Modified Hubble plus modified Hubble
	1 for cluster')
	  CALL MESSAGE('     SI = Sum of K/r and K''/r**2')
	  CALL MESSAGE('     T  = Tapered Hubble')
	  CALL MESSAGE('     V  = de Vaucouleurs')
	  CALL MESSAGE('     VC = de Vaucouleurs plus Hubble for cluster')
	  CALL MESSAGE('     E3 = Extrafocal photometry of E3 galaxies')
	  CALL USRINP(%REF(CHOICE),1,'A',6,1,'CHOICE=',' ')
	  CALL ALPHA(%REF(CHOICE),1,6,LX,NF,NR,2)
	END DO
C  Find out if the user wishes to convolve the surface brightness law
  56	CONVOLVE=.TRUE.
	CALL MESSAGE('Do you wish to convolve the surface brightness '//
	1'law with a smearing')
	CALL USRINP(CONVOLVE,1,'L',4,1,'CONVOLVE=',
	1'distribution?  (default = Y)')
	IF (CONVOLVE) THEN
	  CALL MESSAGE('Open the file containing the smearing function')
	  DCNAME=FILENAMESMEAR !  Sets up default in COMMON block
	  CALL OPENCF(22,1,'FILE_22=',0)
	  FILENAMESMEAR=DCNAME
	  CALL USRINP(%REF(ID2),1,'A',6,1,'ID=',
	1 'Enter the ID of the smearing object:  (default = last '//
	2 'smearing entry)')
	  CALL ALPHA(%REF(ID2),1,6,LX,NF,NR,2)
	  CALL RDDATFL1B(22,ID2,SMEARREAD,SMEARRADIUS,MAXDIAG,NPOINTS_S)
C  Set up SMEAR such that the index equals the corresponding radius plus 1.
	  DO I=1,NPOINTS_S
	    CALL INTERP1D(SMEARRADIUS,SMEARREAD,NPOINTS_S,REAL(I-1),SMEAR(I))
	  END DO
	  CONVOLVEFACTOR=75.0
c	  CALL USRINP(CONVOLVEFACTOR,1,'F',4,1,'CONVOLVE_FACTOR=',
c	1 'Enter the convolving factor:  (default = 75.0)')
	END IF
C  Now get the range of data to be fit
	WRITE(6,1150) RADIUS(1,1),RADIUS(NPOINTS_FILE,1)
1150	FORMAT(' The radii in the file range from ',F4.1,' to ',F6.1)
	WRITE(7,1150)
	CALL MESSAGE('Enter the starting and ending radii for the '//
	1'least squares fit:')
	RADII_MINMAX(1)=RADIUS(1,1)
	RADII_MINMAX(2)=RADIUS(NPOINTS_FILE,1)
	CALL USRINP(RADII_MINMAX,2,'F',4,1,'RADII_MINMAX=','(default = the '//
	1'entire range of the data file)')
	IF (.NOT.CONVOLVE.AND.RADII_MINMAX(1).EQ.0.0) THEN
	  CALL MESSAGE('Do you wish to adjust the first data point to '//
	1 'a radius of')
	  ADJUST=.TRUE.
	  CALL USRINP(ADJUST,1,'L',1,1,'ADJUST=',
	1 '0.25 pixesl?  (default = Y)')
	END IF
C  Get the number of points, allowing for RADIUS to be variably spaced.
C  Even if RADIUS is evenly spaced, NPOINTS_FIT is not necessarily equal
C  to NPOINTS_FILE because we may choose to fit only a few points from
C  the file.
	NPOINTS_FIT=0
	DO I=1,NPOINTS_FILE
	  IF (RADIUS(I,1).GE.RADII_MINMAX(1).AND.RADIUS(I,1).LE.
	1 RADII_MINMAX(2)) NPOINTS_FIT=NPOINTS_FIT+1
	END DO
C  Get the range of smearing if convolution is desired
	IF (CONVOLVE) THEN
	  RADIUS_C=RADII_MINMAX(2)
	  CALL MESSAGE('Enter the outer radius to where convolution is '//
	1 'to be done:')
	  CALL USRINP(RADIUS_C,1,'F',4,1,'RADIUS=',
	1 '(default = previous outer fitting radius')
	  NPOINTS_C=RADIUS_C+1 !  Corrects for the point at zero radius
C  This should be ammended in case the data file does not start at the center
	END IF
C  Now convert the radii to indexes
	DO I=1,NPOINTS_FIT
	  IF (RADIUS(I,1).LE.RADII_MINMAX(1)) IRADII(1)=I
	  IF (RADIUS(I,1).LE.RADII_MINMAX(2)) IRADII(2)=I
	END DO
	IF (ADJUST) THEN
	  RADIUS(IRADII(1),1)=0.25
	  XPLOT(1)=0.25 !  For NONLINFIT 
	END IF
	IF (CHOICE.EQ.'G') THEN
	  CALL MESSAGE('Enter the estimates for the central surface
	1 brightness (ADU), the sigma')
	  CALL MESSAGE('(pixels), and the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M') THEN
	  CALL MESSAGE('Enter the estimates for the central surface '//
	1 'brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), and the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'MC') THEN
	  CALL MESSAGE('Enter the estimates for the central surface '//
	1 'brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), the cluster central surface brightness, '//
	1 '(ADU), the cluster Hubble "a"')
	  CALL MESSAGE('(pixels), and the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
c	ELSE IF (CHOICE.EQ.'ML') THEN
c	  CALL MESSAGE('Enter the fixed values for the central surface '//
c	1 'brightness (ADU), the Hubble "a"')
c	  CALL MESSAGE('(pixels), the sky level (ADU), and the fixed and '//
c	1 'linear terms')
c	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
c	1 'for the color gradient:')
c	ELSE IF (CHOICE.EQ.'MR') THEN
c	  CALL MESSAGE('Enter the fixed values for the central surface '//
c	1 'brightness (ADU), the Hubble "a"')
c	  CALL MESSAGE('(pixels), the sky level (ADU), and the fixed and '//
c	1 'linear terms')
c	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
c	1 'for the color gradient:')
	ELSE IF (CHOICE.EQ.'HV') THEN
	  CALL MESSAGE('Enter the estimates for the central surface '//
	1 'brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), the power and the sky '//
	1 'level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'I') THEN
	  CALL MESSAGE('Enter the estimates for the multiplying factor '//
	1 '(ADU pixels) and the')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
	1 'sky level (ADU):')
c	ELSE IF (CHOICE.EQ.'IX') THEN
c	  CALL MESSAGE('Enter the estimates for the multiplying '//
c	1 'factor (ADU pixels), the power of the')
c	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
c	1 'radius, and the sky level (ADU):')
	ELSE IF (CHOICE.EQ.'K') THEN
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
	1 'Enter the estimate for the constant value:')
	ELSE IF (CHOICE.EQ.'L') THEN
	  CALL MESSAGE('Enter the estimates for the constant and the '//
	1 'slope:')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'SI') THEN
	  CALL MESSAGE('Enter the estimates for the two multiplying '//
	1 'factors (ADU pixels) and the')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',
	1 'sky level (ADU):')
	ELSE IF (CHOICE.EQ.'T') THEN
	  CALL MESSAGE('Enter the estimates for the central surface '//
	1 'brightness (ADU), the Hubble "a"')
	  CALL MESSAGE('(pixels), the taper "b" (pixels), the taper '//
	1 'power, and the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'V') THEN
	  CALL MESSAGE('Enter the estimates for the surface brightness '//
	1 'at 1/2 the total light')
	  CALL MESSAGE('(ADU), the corresponding radius (pixels), and '//
	1 'the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'VC') THEN
	  CALL MESSAGE('Enter the estimates for the surface brightness '//
	1 'at 1/2 the total light')
	  CALL MESSAGE('(ADU), the corresponding radius (pixels), the '//
	1 'cluster central surface')
	  CALL MESSAGE('brightness, (ADU), the cluster Hubble "a" '//
	1 '(pixels), and the sky level (ADU):')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	ELSE IF (CHOICE.EQ.'E3') THEN
	  CALL MESSAGE('Enter the estimates for the total magnitude and '//
	1 'the Hubble a')
	  CALL USRINP(PARAMETRS,MAXNUMPARAM,'F',8,0,'PARAMETERS=',' ')
	END IF
	NUMPARAM=NELD(PARAMETRS,MAXNUMPARAM)
C  Get the weighting scheme
  67	WEIGHT_SCHEME='R'
	CALL MESSAGE('Select the weighting scheme:')
	CALL MESSAGE('     R = Weight equal to circumference/(noise)**2 '//
	1'(default)')
c	CALL MESSAGE('     S = Weight equal to the square root of '//
c	1'the circumference')
	CALL MESSAGE('     N = Weight equal to 1/(noise)**2')
	CALL MESSAGE('     P = Weight equal to weight from file/'//
	1'(constant*noise**2)')
	CALL USRINP(%REF(WEIGHT_SCHEME),1,'A',1,1,'WEIGHT_SCHEME=',' ')
	IF (WEIGHT_SCHEME.NE.'R'.AND.WEIGHT_SCHEME.NE.'S'.AND.
	1WEIGHT_SCHEME.NE.'N'.AND.WEIGHT_SCHEME.NE.'P') GO TO 67
C  Open the file for the weights
	IF (WEIGHT_SCHEME.EQ.'P') THEN
	  CALL MESSAGE('Open the file containing the weights')
	  DCNAME=FILENAMEWEIGHTS
	  CALL OPENCF(23,1,'FILE_23=',0)
	  FILENAMEWEIGHTS=DCNAME
	  REWIND 23
	  READ(23,1201)
1201	  FORMAT(' RADIUS(PIXELS) EXPOSURE(SECONDS)')
	  DO I=1,MAXDIAG
	    READ(23,1202,END=68) R,FILEWEIGHT(I)
1202	    FORMAT(' ',3X,F7.2,8X,F9.3)
	  END DO
  68	  CALL MESSAGE('Enter the constant to normalize the weights '//
	1 'from the file;  this should be')
	  CALL MESSAGE('the exposure of the initial file from which the '//
	1 'weights were calculated:')
	  CONSTANT=2700.0
	  CALL USRINP(CONSTANT,1,'F',4,1,'CONSTANT=',
	1 '(default = 2700')
	END IF
C  Get the readout noise in order to compute the weights
	READOUT(1)=63.9
	READOUT(2)=21.3
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  CALL MESSAGE('Enter the readout noise in electrons and the '//
	1 'gain in electrons per ADU:')
	  CALL USRINP(READOUT,2,'F',4,1,'NOISE_PARAMETERS=',
	1 '(defaults = 63.9 21.3)')
	END IF
	SUMWEIGHT=0.0D0
	DO N=IRADII(1),IRADII(2)
C  Compute the weights for NONLINFIT based on readout noise and shot noise
C  and the number of pixels in each contour.  Then convert to ADU.
	  IF (WEIGHT_SCHEME.EQ.'N'.OR.WEIGHT_SCHEME.EQ.'P'.OR.
	1 WEIGHT_SCHEME.EQ.'R') THEN
	    WEIGHT(N,1)=READOUT(2)**2/
	1   (READOUT(1)**2+SURFBRIGHTOB(N,1)*READOUT(2))
	    IF (WEIGHT_SCHEME.EQ.'R') THEN
	      WEIGHT(N,1)=WEIGHT(N,1)*MAX(1.0,TWOPI*SNGL(RADIUS(N,1)))
	    ELSE IF (WEIGHT_SCHEME.EQ.'N') THEN
	      WEIGHT(N,1)=WEIGHT(N,1)
	    ELSE IF (WEIGHT_SCHEME.EQ.'P') THEN
	      WEIGHT(N,1)=WEIGHT(N,1)*FILEWEIGHT(N)/CONSTANT
	    END IF
	  ELSE IF (WEIGHT_SCHEME.EQ.'S') THEN !  Square root of radius
					      !  only
	    WEIGHT(N,1)=SQRT(RADIUS(N,1))
	  END IF
	  SUMWEIGHT=WEIGHT(N,1)+SUMWEIGHT !  For use below
	END DO
	NPLOTPOINTS=NPOINTS_FIT
C
	CALL NONLINFIT(RADIUS,MAXDIAG,DIMENSN_IND,SURFBRIGHTOB,
	1DIMENSN_DATA,WEIGHT,IRADII,FUNCAT,CHOICE,CONSTANTTERMS,
	2PARAMETRS,MAXNUMPARAM,NUMPARAM,ITERATION_SCHEME,TOLERANCE,
	3INTERMEDIATE,CONVOLVE,COVARIANCE,MEANERROR,MEANERRORMEANERROR,
	4REDCHISQ,AVREDCHISQ,XPLOT,DATPLOT,NPLOTPOINTS,RESIDUALS)
C
	CALL MESSAGE(' ')
	CALL MESSAGE('Answer')
	WRITE(6,1300) MEANERROR(1),MEANERRORMEANERROR(1)
1300	FORMAT(' Mean error for an observation of average weight ',
	1'= '/G18.6,  '   +-',G18.6)
	WRITE(7,1300) MEANERROR(1),MEANERRORMEANERROR(1)
C  Calculate the residuals in magnitudes and load the plot arrays
	DO N=IRADII(1),IRADII(2)
	  OBMINUSCALC(N,1)=SURFBRIGHTOB(N,1)-DATPLOT(N)
	  IF (CHOICE.EQ.'G'.OR.CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'
	1 .OR.CHOICE.EQ.'V') SKY=PARAMETRS(3)
			!  Otherwise it is the minimum from above
	  IF (SURFBRIGHTOB(N,1)-SKY.NE.0.0.AND.DATPLOT(N)-SKY.NE.0.0) THEN
	    OBMINUSCALC(N,2)=ABS(2.5*LOG10(ABS((SURFBRIGHTOB(N,1)-SKY)/
	1   (DATPLOT(N)-SKY))))
	  ELSE
	    OBMINUSCALC(N,2)=0.0
	  END IF
	  SIGMASUM_MAG=OBMINUSCALC(N,2)**2*WEIGHT(N,1)+SIGMASUM_MAG
	END DO
	AVWEIGHT=SUMWEIGHT/NPOINTS_FIT
	SIGMA_MAG=SQRT(SIGMASUM_MAG/((NPOINTS_FIT-NUMPARAM)*AVWEIGHT))
	SIGMASIGMA_MAG=SIGMA_MAG/SQRT(2.0*(NPOINTS_FIT-NUMPARAM))
C	
	IF (CHOICE.NE.'E3') THEN
	  WRITE(6,1305) SIGMA_MAG,SIGMASIGMA_MAG
1305	  FORMAT('                         and in magnitudes it is',
	1 '= '/G18.6,  '   +-',G18.6)
	  WRITE(7,1305) SIGMA_MAG,SIGMASIGMA_MAG
	END IF
C  Compute the expected mean error for a perfect fit
	IF (WEIGHT_SCHEME.EQ.'R'.OR.WEIGHT_SCHEME.EQ.'N'.OR.
	1WEIGHT_SCHEME.EQ.'P') THEN
	  EXPCT_SIGMA=SQRT(NPOINTS_FIT/SUMWEIGHT)
	  EXPCT_SIGMASIGMA=EXPCT_SIGMA/SQRT(2.0*(NPOINTS_FIT-NUMPARAM))
	  CALL MESSAGE('For a perfect fit:')
	  WRITE (6,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	  WRITE (7,1300) EXPCT_SIGMA,EXPCT_SIGMASIGMA
	END IF
C  Print the reduced chi squared
	IF (WEIGHT_SCHEME.NE.'S') THEN
	  WRITE (6,1415) REDCHISQ(1)
	  WRITE (7,1415) REDCHISQ(1)
1415	  FORMAT(' The reduced chi squared = ',F10.4)
	END IF
C  Print the average weight
	WRITE (6,1410) AVWEIGHT
	WRITE (7,1410) AVWEIGHT
1410	FORMAT(' The average weight = ',F15.6)
	DO I=1,NUMPARAM
	  SIGMAPARAM(I)=SQRT(COVARIANCE(I,I))
	END DO
	IF (CHOICE.EQ.'G') THEN
1309	  FORMAT(' Central surface brightness = ',F15.4,'   +-',F15.4/
	1 ' Sigma = ',F15.4,'   +-',F15.4)
	  WRITE (6,1309) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1309) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  SKY=PARAMETRS(3) !  For the integration below
1310	  FORMAT(' Sky surface brightness = ',F15.4,'   +-',F15.4)
	  WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	  WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	ELSE IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'MC') THEN
1311	  FORMAT(' Central surface brightness = ',F15.4,'   +-',F15.4/
	1 ' Hubble "a" = ',F15.4,'   +-',F15.4)
	  WRITE (6,1311) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1311) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  IF (CHOICE.EQ.'MC') THEN
1312	    FORMAT(' Cluster central surface brightness = ',F15.4,'  +-',
	1   F15.4/' Cluster Hubble "a" = ',F15.4,'   +-',F15.4)
	    WRITE (6,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	    WRITE (7,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	    SKY=PARAMETRS(5) !  For the integration below
	    WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
	    WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	  ELSE
	    SKY=PARAMETRS(3) !  For the integration below
	    WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	    WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	  END IF
c	ELSE IF (CHOICE.EQ.'ML'.OR.CHOICE.EQ.'MR') THEN
c	  SKY=PARAMETRS(3)
c1314	  FORMAT(' Constant term = ',F15.4,'   +-',F15.4/
c	1 ' Linear term = ',F15.4,'   +-',F15.4)
c	  WRITE (6,1314) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
c	  WRITE (7,1314) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	ELSE IF (CHOICE.EQ.'HV') THEN
1315	  FORMAT(' Central surface brightness = ',F15.4,'   +-',G15.4/
	1 ' Hubble "a" = ',F15.4,'   +-',G15.4/
	2 ' Power = ',F15.4,'  +-',G15.4)
	  WRITE (6,1315) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	  WRITE (7,1315) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	  SKY=PARAMETRS(4) !  For the integration below
	  WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
	  WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
	ELSE IF (CHOICE.EQ.'I') THEN
1320      FORMAT(' Multiplying factor = ',F15.4,'  +-',G15.4)
	  WRITE (6,1320) PARAMETRS(1),SIGMAPARAM(1)
	  WRITE (7,1320) PARAMETRS(1),SIGMAPARAM(1)
	  SKY=PARAMETRS(2)
	  WRITE (6,1310) PARAMETRS(2),SIGMAPARAM(2)
	  WRITE (7,1310) PARAMETRS(2),SIGMAPARAM(2)
	ELSE IF (CHOICE.EQ.'IX') THEN
1321	  FORMAT(' Multiplying factor = ',F15.4,'  +-',F15.4/
	1 ' Power of radius = ',F15.4,'  +-',F15.4)
	  WRITE (6,1321) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1321) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  SKY=PARAMETRS(3)
	  WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	  WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	ELSE IF (CHOICE.EQ.'K') THEN
1330	  FORMAT(' Constant = ',F15.6,'   +-',G15.6)
	  WRITE (6,1330) (PARAMETRS(I),SIGMAPARAM(I),I=1,1)
	  WRITE (7,1330) (PARAMETRS(I),SIGMAPARAM(I),I=1,1)
	ELSE IF (CHOICE.EQ.'L') THEN
1331	  FORMAT(' Constant = ',F15.6,'   +-',G15.6/' Slope = ',G15.6,
	1 '   +-',G15.6)
	  WRITE (6,1331) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1331) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	ELSE IF (CHOICE.EQ.'SI') THEN
1322	  FORMAT(' First order multiplying factor = ',F15.4,' +-',
	1 F15.4/' Second order multiplying factor = ',F15.4,' +-',
	2 F15.4)
	  WRITE (6,1322) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1322) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  SKY=PARAMETRS(3)
	  WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	  WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	ELSE IF (CHOICE.EQ.'T') THEN
1340	  FORMAT(' Central surface brightness = ',F15.4,'   +-',F15.4/
	1 ' Hubble "a" = ',F15.4,'   +-',F15.4/' Taper = ',F15.4,
	2 '   +-',F15.4)
	  WRITE (6,1340) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	  WRITE (7,1340) (PARAMETRS(I),SIGMAPARAM(I),I=1,3)
	  SKY=PARAMETRS(4)
	  WRITE (6,1310) PARAMETRS(4),SIGMAPARAM(4)
	  WRITE (7,1310) PARAMETRS(4),SIGMAPARAM(4)
c	ELSE IF (CHOICE.EQ.'TX') THEN
c1341	  FORMAT(' Central surface brightness = ',F15.4,'   +-',F15.4/
c	1 ' Hubble "a" = ',F15.4,'   +-',F15.4/' Taper = ',F15.4,
c	2 '   +-',F15.4/' Taper power = ',F15.4,'   +-',F15.4)
c	  WRITE (6,1341) (PARAMETRS(I),SIGMAPARAM(I),I=1,4)
c	  WRITE (7,1341) (PARAMETRS(I),SIGMAPARAM(I),I=1,4)
c	  SKY=PARAMETRS(5)
c	  WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
c	  WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	ELSE IF (CHOICE.EQ.'V'.OR.CHOICE.EQ.'VC') THEN
1350	  FORMAT(' Effective surface brightness = ',F15.4,'   +-',
	1 F15.4/' Effective radius = ',F15.4,'   +-',F15.4)
	  WRITE (6,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1350) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  IF (CHOICE.EQ.'VC') THEN
	    WRITE (6,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	    WRITE (7,1312) (PARAMETRS(I),SIGMAPARAM(I),I=3,4)
	    SKY=PARAMETRS(5) !  Used below in integration
	    WRITE (6,1310) PARAMETRS(5),SIGMAPARAM(5)
	    WRITE (7,1310) PARAMETRS(5),SIGMAPARAM(5)
	  ELSE
	    SKY=PARAMETRS(3) !  Used below in integration
	    WRITE (6,1310) PARAMETRS(3),SIGMAPARAM(3)
	    WRITE (7,1310) PARAMETRS(3),SIGMAPARAM(3)
	  END IF
	ELSE IF (CHOICE.EQ.'E3') THEN
1363	  FORMAT(' Total magnitude = ',F9.4,'   +-',
	1 F9.4/' Radius-parameter = ',F9.5,'   +-',F9.5)
	  WRITE (6,1363) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	  WRITE (7,1363) (PARAMETRS(I),SIGMAPARAM(I),I=1,2)
	END IF
	DO J=1,NUMPARAM
	  DO I=1,J
	    K=K+1
	      WRITE (6,1400) I,J,COVARIANCE(I,J)
	      WRITE (7,1400) I,J,COVARIANCE(I,J)
1400	      FORMAT(' Covariance(',I1,',',I1,') = ',G20.8)
	  END DO
	END DO
C
C  Compute the integral of the surface brightness distribution
C  using the actual data out to the ending input radius and the 
C  function just solved for from there to infinity (actually to
C  where the answer doesn't change by more than 1 part in 10(10)
C  for a one pixel increase in radius).  The integration in the center
C  of the object will be accurate if the data file was prepared from
C  a two-dimensional array by options EA or PR, but not so accurate if
C  the data file was created from a formula as from option FG.
C
	IF (CHOICE.NE.'E3') THEN
	  ADUINTEGRAL1=0.0 !  The integration must be zero at the start
	  FORESHORT=SQRT(1.0-ECCENTRICITY(2)**2)
	  ADUINTEGRAL1=(SURFBRIGHTOB(1,1)-SKY)/PI+(SURFBRIGHTOB(2,1)-
	1 SKY)*2*FORESHORT
	  DO 520 I=3,IRADII(2)
	    FORESHORT=SQRT(1.0-ECCENTRICITY(I)**2)
	    ADUINTEGRAL1=ADUINTEGRAL1+(SURFBRIGHTOB(I,1)-SKY)*2*(I-1)*
	1   FORESHORT
 520	  CONTINUE
	  ADUINTEGRAL1=ADUINTEGRAL1*PI
	  ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL1))
1420	  FORMAT(' The integral from radius = ',F4.1,' to radius = ',
	1 F6.1,' = ',F20.6)
	  CALL MESSAGE(' ')
	  WRITE(6,1420) RADII_MINMAX(1),RADII_MINMAX(2),ADUINTEGRAL1
	  WRITE(7,1420) RADII_MINMAX(1),RADII_MINMAX(2),ADUINTEGRAL1
1430	  FORMAT(' In magnitudes it is ',F11.6)
	  WRITE(6,1430) ADUMAGINTEGRAL
	  WRITE(7,1430) ADUMAGINTEGRAL
	  ISTARTPLOT=IRADII(1) !  To be used in plotting residuals
	  N=IRADII(2)+1 !  Next available array location to be used
			!  in loop for integration
	  IRADII(1)=N
	  IRADII(2)=IRADII(1)
	  RADII_MINMAX(1)=RADII_MINMAX(2)+1
	  RADIUS(N,1)=RADII_MINMAX(1) !  The radius at which the integration
			       !  begins
	  STARTRADIUS=RADIUS(N,1)
	  TEST=1.0 !  Initialization
	  ADUINTEGRAL2=0.0
	  IF (CHOICE.NE.'H'.AND.CHOICE.NE.'ML'.AND.CHOICE.NE.'MR'.AND.CHOICE
	1 .NE.'I'.AND.CHOICE.NE.'IX'.AND.CHOICE
	2 .NE.'K'.AND.CHOICE.NE.'L'.AND.CHOICE.NE.'SI') THEN
C  Now integrate using the formula.  Note that N is constant here
	    EVALUATE=.TRUE.
	    GO=.TRUE.
C  Integrate only the galaxy for the following choices
	    IF (CHOICE.EQ.'MC') CHOICE='M'
	    IF (CHOICE.EQ.'VC') CHOICE='V'
	    CONVOLVE=.FALSE.
	    DO WHILE (GO)
C  Use the array SURFBRIGHTOB to return the evaluated surface brightness
	      CALL NLFUN1(RADIUS,MAXDIAG,DIMENSN_IND,PARAMETRS,CHOICE,
	1     CONVOLVE,IRADII,SURFBRIGHTOB,DIMENSN_DATA)
	      DELTARADIUS=1.0+0.01*(RADIUS(N,1)/STARTRADIUS)
	      TEST=TWOPI*(SURFBRIGHTOB(N,1)-SKY)*(RADIUS(N,1)-1.0+
	1     (DELTARADIUS+1.0)/2.0)*DELTARADIUS*FORESHORT
	      ADUINTEGRAL2=ADUINTEGRAL2+TEST
	      RADIUS(N,1)=RADIUS(N,1)+DELTARADIUS
	      IF (TEST/(ADUINTEGRAL1+ADUINTEGRAL2).LT.1E-10) GO=.FALSE.
	    END DO
	    RADII_MINMAX(2)=RADIUS(N,1)
1440	    FORMAT(' The integral from radius =',F6.1,' to the limit of',
	1   F12.1,' =',F17.6)
	    WRITE(6,1440) RADII_MINMAX(1),RADII_MINMAX(2),ADUINTEGRAL2
	    WRITE(7,1440) RADII_MINMAX(1),RADII_MINMAX(2),ADUINTEGRAL2
	    IF (ADUINTEGRAL2.GT.0.0) THEN
	      ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL2))
	    ELSE
	      ADUINTEGRAL2=0.0
	      ADUMAGINTEGRAL=100.0
	    END IF
	    WRITE(6,1430) ADUMAGINTEGRAL
	    WRITE(7,1430) ADUMAGINTEGRAL
	    ADUINTEGRAL1=ADUINTEGRAL1+ADUINTEGRAL2
	    ADUMAGINTEGRAL=30.0-2.5*SNGL(DLOG10(ADUINTEGRAL1))
1450	    FORMAT(' The total is ',F20.6,' and in magnitudes it is ',
	1   F11.6)
	    WRITE(6,1450) ADUINTEGRAL1,ADUMAGINTEGRAL
	    WRITE(7,1450) ADUINTEGRAL1,ADUMAGINTEGRAL
	    CALL MESSAGE(' ')
	  END IF
	END IF
	PLOTD=.TRUE.
	CALL USRINP(PLOTD,1,'L',1,1,'PLOT=',
	1'Do you wish to plot the residuals?  (default = Y)')
	IF (PLOTD) THEN
	  XLABEL='Radius (pixels)'
	  YLABEL='Observed - Calculated (ADU)'
	  TITLE='Residuals'
C  Reset the array
	  DO I=MAXDIAG-ISTARTPLOT+1-3,MAXDIAG-ISTARTPLOT+1
	    XPLOT(I)=0
	  END DO
	  IF (ADJUST) XPLOT(ISTARTPLOT)=0.25
C  XPLOT was loaded with the radii from the file;  OBMINUSCALC was loaded
C  with the residuals returned from NONLINFIT.
	  CALL MESSAGE('Residuals in linear units')
	  CALL XYPLOT(XPLOT(ISTARTPLOT),OBMINUSCALC(ISTARTPLOT,1),
	1 MAXDIAG-ISTARTPLOT+1,NPLOTS,NPLOTPOINTS,LINVECT,INC,LINTYPE,
	2 NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,YLABEL,
	3 TITLE,' ',.FALSE.)
	  CALL RESETTEK
	  IF (CHOICE.NE.'E3') THEN
	    YLABEL='Observed - Calculated (magnitudes)'
C  Reset the array
	    DO I=MAXDIAG-ISTARTPLOT+1-3,MAXDIAG-ISTARTPLOT+1
	      XPLOT(I)=0
	    END DO
	    CALL MESSAGE('Residuals in units of magnitudes')
	    CALL XYPLOT(XPLOT(ISTARTPLOT),OBMINUSCALC(ISTARTPLOT,2),
	1   MAXDIAG-ISTARTPLOT+1,NPLOTS,NPOINTS_FIT,LINVECT,INC,LINTYPE,
	2   NSYMBOL,INVERT,COLOR,INTERACT,CROSSHAIRS,XLABEL,YLABEL,
	3   TITLE,' ',.FALSE.)
	  END IF
	  CALL RESETTEK
	END IF
	STORERESULTS=.FALSE.
	IF (CHOICE.EQ.'H'.OR.CHOICE.EQ.'M'.OR.CHOICE.EQ.'V') THEN
	  CALL USRINP(STORERESULTS,1,'L',1,1,'STORE=','Do you wish to '//
	1 'store the results?  (default = N)')
	END IF
	IF (STORERESULTS.AND..NOT.STOREOPENED) THEN
	  DCNAME=FILENAMERESULTS
	  CALL OPENCF(24,0,'FILE_24=',0)
	  FILENAMERESULTS=DCNAME
	  STOREOPENED=.TRUE.
C  If this is a new file, write the header
	  IF (.NOT.OFIL) THEN
	    WRITE(24,2200)
2200	    FORMAT(' ','ID',4X,'SURFACE BRIGHTNESS',1X,'RADIUS PARAMETER',
	1   1X,'TOTAL MAGNITUDE',1X,'MEAN ERROR',1X,'MEAN ERROR SB',1X,
	2   'MEAN ERROR RP',3X,'SKY',3X,'M.E. SKY')
	  END IF
 	END IF
	IF (STORERESULTS) THEN
	  WRITE(24,2201) ID,(PARAMETRS(I),I=1,2),ADUMAGINTEGRAL,SIGMAADU,
	1 (SIGMAPARAM(I),I=1,2),PARAMETRS(3),SIGMAPARAM(3)
2201	  FORMAT(' ',A6,F15.3,5X,F11.5,8X,F7.3,6X,F7.3,4X,F9.3,5X,F9.4,3X,
	1 F9.3,2X,F6.3)
	END IF
	MORE=.FALSE.
	CLOSE (UNIT=22) !  Closes the smearing file
	CLOSE (UNIT=23) !  Closes the file for the weights
	CALL USRINP(MORE,1,'L',1,1,'MORE=',
	1'More objects to analyze in this file?   (default = N)')
	IF (MORE) GO TO 2
	CLOSE(UNIT=21) !  Closes the data file
	IF (STORERESULTS) CLOSE(UNIT=24)
	RETURN
	END
